#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
OffsiteMiddleware 中间件
用于过滤掉不在指定域名范围内的请求
"""
import re
from urllib.parse import urlparse

from crawlo.logging import get_logger
from crawlo.exceptions import IgnoreRequestError


class OffsiteMiddleware:
    """
    OffsiteMiddleware 中间件
    用于过滤掉不在指定域名范围内的请求，防止爬虫爬取到不相关的网站
    """

    def __init__(self, stats, allowed_domains=None):
        self.logger = get_logger(self.__class__.__name__)
        self.stats = stats
        self.allowed_domains = allowed_domains or []

    @classmethod
    def create_instance(cls, crawler):
        """
        创建中间件实例
        从爬虫设置中获取允许的域名列表
        """
        # 优先使用 Spider 实例的 allowed_domains，回退到全局设置中的 ALLOWED_DOMAINS
        allowed_domains = []
        
        # 检查当前爬虫实例是否有 allowed_domains 属性
        if hasattr(crawler, 'spider') and crawler.spider and hasattr(crawler.spider, 'allowed_domains'):
            allowed_domains = getattr(crawler.spider, 'allowed_domains', [])
        
        # 如果 Spider 实例没有设置 allowed_domains，则从全局设置中获取
        if not allowed_domains:
            allowed_domains = crawler.settings.get_list('ALLOWED_DOMAINS')
        
        # 如果没有配置允许的域名，则禁用此中间件
        if not allowed_domains:
            from crawlo.exceptions import NotConfiguredError
            raise NotConfiguredError("未配置ALLOWED_DOMAINS，OffsiteMiddleware已禁用")
            
        o = cls(
            stats=crawler.stats,
            allowed_domains=allowed_domains
        )
        
        # 编译域名正则表达式以提高性能
        o._compile_domains()
        
        # 使用中间件自己的logger而不是crawler.logger
        o.logger.debug(f"OffsiteMiddleware 已启用，允许的域名: {allowed_domains}")
        return o

    def _compile_domains(self):
        """
        编译域名正则表达式
        """
        self._domain_regexes = []
        for domain in self.allowed_domains:
            # 转义域名中的特殊字符
            escaped_domain = re.escape(domain)
            # 创建匹配域名的正则表达式（支持子域名）
            regex = re.compile(r'(^|.*\.)' + escaped_domain + '$', re.IGNORECASE)
            self._domain_regexes.append(regex)

    def _is_offsite_request(self, request):
        """
        判断请求是否为站外请求
        """
        try:
            parsed_url = urlparse(request.url)
            hostname = parsed_url.hostname
            
            if not hostname:
                return True  # 无效URL
                
            # 检查是否匹配允许的域名
            for regex in self._domain_regexes:
                if regex.match(hostname):
                    return False  # 匹配允许的域名
                    
            return True  # 不匹配任何允许的域名
        except Exception:
            # URL解析失败，视为站外请求
            return True

    async def process_request(self, request, spider):
        """
        处理请求，过滤站外请求
        """
        if self._is_offsite_request(request):
            # 记录被过滤的请求
            self.stats.inc_value('offsite_request_count')
            
            # 记录被过滤的域名
            try:
                parsed_url = urlparse(request.url)
                hostname = parsed_url.hostname or "unknown"
                self.stats.inc_value(f'offsite_request_count/{hostname}')
            except:
                self.stats.inc_value('offsite_request_count/invalid_url')
            
            self.logger.debug(f"过滤站外请求: {request.url}")
            
            # 抛出异常以忽略该请求
            raise IgnoreRequestError(f"站外请求被过滤: {request.url}")
            
        return None

    def process_exception(self, request, exception, spider):
        """
        处理异常
        """
        # 如果是IgnoreRequestError且是我们产生的，则处理它
        if isinstance(exception, IgnoreRequestError) and "站外请求被过滤" in str(exception):
            self.logger.debug(f"已过滤站外请求: {request.url}")
            return True  # 表示异常已被处理
        return None


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
增强错误处理工具综合测试
测试 ErrorContext, DetailedException, EnhancedErrorHandler 的更多功能
"""
import sys
import os
import unittest
from unittest.mock import Mock, patch, MagicMock
import asyncio
import traceback

# 添加项目根目录到 Python 路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.utils.enhanced_error_handler import (
    EnhancedErrorHandler, 
    ErrorContext, 
    DetailedException, 
    handle_exception
)


class TestErrorContext(unittest.TestCase):
    """错误上下文测试"""

    def test_error_context_initialization(self):
        """测试错误上下文初始化"""
        context = ErrorContext(
            context="测试上下文",
            module="test_module",
            function="test_function"
        )
        
        self.assertEqual(context.context, "测试上下文")
        self.assertEqual(context.module, "test_module")
        self.assertEqual(context.function, "test_function")
        self.assertIsNotNone(context.timestamp)
        
    def test_error_context_string_representation(self):
        """测试错误上下文字符串表示"""
        context = ErrorContext(
            context="测试上下文",
            module="test_module",
            function="test_function"
        )
        
        context_str = str(context)
        self.assertIn("Module: test_module", context_str)
        self.assertIn("Function: test_function", context_str)
        self.assertIn("Context: 测试上下文", context_str)
        self.assertIn("Time:", context_str)


class TestDetailedException(unittest.TestCase):
    """详细异常测试"""

    def test_detailed_exception_initialization(self):
        """测试详细异常初始化"""
        context = ErrorContext(context="测试上下文")
        
        exception = DetailedException(
            message="测试异常消息",
            context=context,
            error_code="TEST_001",
            detail1="详细信息1",
            detail2="详细信息2"
        )
        
        self.assertIn("测试异常消息", str(exception))
        self.assertEqual(exception.context, context)
        self.assertEqual(exception.error_code, "TEST_001")
        self.assertEqual(exception.details["detail1"], "详细信息1")
        self.assertEqual(exception.details["detail2"], "详细信息2")
        self.assertIsNotNone(exception.timestamp)
        
    def test_detailed_exception_without_context(self):
        """测试没有上下文的详细异常"""
        exception = DetailedException(
            message="测试异常消息",
            error_code="TEST_002"
        )
        
        self.assertEqual(str(exception), "测试异常消息")
        self.assertIsNone(exception.context)
        self.assertEqual(exception.error_code, "TEST_002")
        self.assertIsNotNone(exception.timestamp)
        
    def test_detailed_exception_string_with_context(self):
        """测试带上下文的详细异常字符串表示"""
        context = ErrorContext(context="测试上下文")
        exception = DetailedException(
            message="测试异常消息",
            context=context
        )
        
        exception_str = str(exception)
        self.assertIn("测试异常消息", exception_str)
        self.assertIn("测试上下文", exception_str)
        
    def test_get_full_details(self):
        """测试获取完整详情"""
        context = ErrorContext(context="测试上下文")
        exception = DetailedException(
            message="测试异常消息",
            context=context,
            error_code="TEST_003",
            detail1="详细信息1"
        )
        
        details = exception.get_full_details()
        self.assertIn("测试异常消息", details["message"])
        self.assertEqual(details["error_code"], "TEST_003")
        self.assertIn("测试上下文", details["context"])
        self.assertEqual(details["details"]["detail1"], "详细信息1")
        self.assertEqual(details["exception_type"], "DetailedException")


class TestEnhancedErrorHandler(unittest.TestCase):
    """增强错误处理器测试"""

    def setUp(self):
        """测试前准备"""
        self.handler = EnhancedErrorHandler("test_logger", "ERROR")
        
    def test_handler_initialization(self):
        """测试处理器初始化"""
        self.assertEqual(len(self.handler.error_history), 0)
        self.assertEqual(self.handler.max_history_size, 100)
        
    def test_handle_error_without_raising(self):
        """测试不抛出异常的错误处理"""
        try:
            raise ValueError("测试错误")
        except Exception as e:
            # 处理错误但不重新抛出
            error_details = self.handler.handle_error(e, raise_error=False, log_error=False)
            
            # 验证返回的错误详情
            self.assertIsInstance(error_details, dict)
            self.assertEqual(error_details["exception_type"], "ValueError")
            self.assertEqual(error_details["message"], "测试错误")
            
    def test_safe_call_success(self):
        """测试安全调用成功"""
        def normal_function(x, y):
            return x + y
            
        result = self.handler.safe_call(normal_function, 1, 2, default_return=0)
        self.assertEqual(result, 3)
        
    def test_safe_call_with_exception(self):
        """测试安全调用异常"""
        def failing_function():
            raise RuntimeError("函数执行失败")
            
        result = self.handler.safe_call(failing_function, default_return="默认值")
        self.assertEqual(result, "默认值")
        
    def test_get_and_clear_error_history(self):
        """测试获取和清空错误历史"""
        # 产生一些错误
        try:
            raise ValueError("错误1")
        except Exception as e:
            self.handler.handle_error(e, raise_error=False, log_error=False)
            
        try:
            raise RuntimeError("错误2")
        except Exception as e:
            self.handler.handle_error(e, raise_error=False, log_error=False)
            
        # 检查历史记录
        history = self.handler.get_error_history()
        self.assertEqual(len(history), 2)
        
        # 清空历史记录
        self.handler.clear_error_history()
        history = self.handler.get_error_history()
        self.assertEqual(len(history), 0)
        
    def test_error_history_size_limit(self):
        """测试错误历史大小限制"""
        # 产生超过100个错误
        for i in range(110):
            try:
                raise ValueError(f"错误{i}")
            except Exception as e:
                self.handler.handle_error(e, raise_error=False, log_error=False)
                
        # 检查历史记录大小限制
        history = self.handler.get_error_history()
        self.assertEqual(len(history), 100)  # 应该限制在100个


class TestHandleExceptionDecorator(unittest.TestCase):
    """异常处理装饰器测试"""

    def test_handle_exception_decorator_sync(self):
        """测试同步函数的异常处理装饰器"""
        @handle_exception(context="测试装饰器", module="test_module", raise_error=False)
        def failing_function():
            raise ValueError("装饰器测试错误")
            
        # 调用应该捕获异常但不抛出
        try:
            result = failing_function()
            self.assertIsNone(result)  # 默认返回None
        except Exception:
            self.fail("异常应该被捕获")
            
    def test_handle_exception_decorator_async(self):
        """测试异步函数的异常处理装饰器"""
        @handle_exception(context="异步测试装饰器", module="test_module", raise_error=False)
        async def async_failing_function():
            raise RuntimeError("异步装饰器测试错误")
            
        # 异步调用
        async def test_async():
            try:
                result = await async_failing_function()
                self.assertIsNone(result)  # 默认返回None
            except Exception:
                self.fail("异常应该被捕获")
                
        # 使用事件循环运行异步函数
        asyncio.run(test_async())
        
    def test_handle_exception_decorator_with_detailed_exception(self):
        """测试装饰器处理详细异常"""
        @handle_exception(context="详细异常测试", module="test_module", raise_error=False)
        def function_with_detailed_exception():
            context = ErrorContext(context="函数内部上下文")
            raise DetailedException("详细异常消息", context=context, error_code="DETAIL_001")
            
        # 调用应该捕获异常
        try:
            result = function_with_detailed_exception()
            self.assertIsNone(result)
        except Exception:
            self.fail("详细异常应该被捕获")


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
集成测试
测试 Crawlo 框架的核心功能集成
"""

import asyncio
import sys
import os
from unittest.mock import Mock, AsyncMock

# 添加项目根目录到路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.config import CrawloConfig
from crawlo.crawler import CrawlerProcess
from crawlo import Spider, Request, Item
from crawlo.extension.memory_monitor import MemoryMonitorExtension


class MockItem(Item):
    """模拟数据项"""
    title = ''
    url = ''


class MockSpider(Spider):
    """模拟爬虫"""
    name = 'mock_spider'
    
    async def start_requests(self):
        """发起模拟请求"""
        yield Request('https://httpbin.org/get', callback=self.parse)
    
    async def parse(self, response):
        """解析响应"""
        item = MockItem(
            title='Test Item',
            url=response.url
        )
        yield item


class MockSettings:
    """模拟设置"""
    def get(self, key, default=None):
        config = {
            'PROJECT_NAME': 'integration_test',
            'LOG_LEVEL': 'WARNING',  # 减少日志输出
            'REDIS_URL': 'redis://127.0.0.1:6379/15',
            'REDIS_HOST': '127.0.0.1',
            'REDIS_PORT': 6379,
            'REDIS_DB': 15,
            'FILTER_CLASS': 'crawlo.filters.aioredis_filter.AioRedisFilter',
            'CUSTOM_PIPELINES': ['crawlo.pipelines.redis_dedup_pipeline.RedisDedupPipeline'],
            'CUSTOM_EXTENSIONS': [
                'crawlo.extension.memory_monitor.MemoryMonitorExtension',
            ],
            'MEMORY_MONITOR_ENABLED': True,
            'MEMORY_MONITOR_INTERVAL': 1,
            'MEMORY_WARNING_THRESHOLD': 95.0,
            'MEMORY_CRITICAL_THRESHOLD': 98.0,
            'CONCURRENT_REQUESTS': 5,
            'DOWNLOAD_DELAY': 0.1,
        }
        return config.get(key, default)
    
    def get_int(self, key, default=0):
        value = self.get(key, default)
        return int(value) if value is not None else default
        
    def get_float(self, key, default=0.0):
        value = self.get(key, default)
        return float(value) if value is not None else default
        
    def get_bool(self, key, default=False):
        value = self.get(key, default)
        if isinstance(value, bool):
            return value
        if isinstance(value, str):
            return value.lower() in ('true', '1', 'yes')
        return bool(value)
        
    def get_list(self, key, default=None):
        value = self.get(key, default or [])
        if isinstance(value, str):
            return [v.strip() for v in value.split(',') if v.strip()]
        return list(value)


async def test_crawler_integration():
    """测试爬虫集成"""
    print("🔍 测试爬虫集成...")
    
    # 创建配置
    config = CrawloConfig.standalone(
        concurrency=2,
        download_delay=0.1,
        LOG_LEVEL='WARNING'
    )
    
    # 添加自定义管道和扩展
    config.set('CUSTOM_PIPELINES', [
        'crawlo.pipelines.console_pipeline.ConsolePipeline',
    ])
    
    config.set('CUSTOM_EXTENSIONS', [
        'crawlo.extension.memory_monitor.MemoryMonitorExtension',
    ])
    
    # 创建爬虫进程
    process = CrawlerProcess(settings=config.to_dict())
    
    # 添加爬虫
    process.crawl(MockSpider)
    
    # 运行测试
    await process.start()
    
    print("   爬虫集成测试完成")


async def test_extension_integration():
    """测试扩展集成"""
    print("🔍 测试扩展集成...")
    
    # 创建模拟爬虫
    mock_crawler = Mock()
    mock_crawler.settings = MockSettings()
    mock_crawler.subscriber = Mock()
    mock_crawler.subscriber.subscribe = Mock()
    
    try:
        # 尝试创建内存监控扩展实例
        extension = MemoryMonitorExtension.create_instance(mock_crawler)
        print("   扩展集成测试完成")
    except Exception as e:
        if "NotConfigured" in str(type(e)):
            print("   扩展未启用（正常情况）")
        else:
            raise e


async def main():
    """主测试函数"""
    print("开始Crawlo框架集成测试...")
    print("=" * 50)
    
    try:
        await test_crawler_integration()
        await test_extension_integration()
        
        print("=" * 50)
        print("所有集成测试通过！")
        
    except Exception as e:
        print("=" * 50)
        print(f"测试失败: {e}")
        import traceback
        traceback.print_exc()
        return 1
    
    return 0


if __name__ == "__main__":
    exit_code = asyncio.run(main())
    sys.exit(exit_code)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
增强版代理中间件测试
==================
测试ProxyMiddleware的代理池和健康检查功能
"""

import asyncio
import json
import sys
import os

# 添加项目根目录到Python路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from unittest.mock import AsyncMock, Mock, patch

from crawlo.middleware.proxy import ProxyMiddleware, Proxy
from crawlo.network.request import Request
from crawlo.network.response import Response
from crawlo.settings.setting_manager import SettingManager


def test_proxy_class():
    """测试Proxy类的基本功能"""
    print("=== 测试Proxy类 ===")
    
    # 创建代理对象
    proxy = Proxy("http://127.0.0.1:8080")
    print(f"初始代理: {proxy.proxy_str}")
    print(f"初始成功率: {proxy.success_rate}")
    print(f"是否健康: {proxy.is_healthy}")
    
    # 测试成功标记
    proxy.mark_success()
    print(f"标记成功后 - 成功率: {proxy.success_rate}, 成功次数: {proxy.success_count}")
    
    # 测试失败标记
    proxy.mark_failure()
    print(f"标记失败后 - 成功率: {proxy.success_rate}, 失败次数: {proxy.failure_count}")
    print(f"是否健康: {proxy.is_healthy}")
    
    # 测试多次失败后健康状态
    for _ in range(5):
        proxy.mark_failure()
    print(f"多次失败后 - 成功率: {proxy.success_rate}, 是否健康: {proxy.is_healthy}")
    
    print("Proxy类测试完成\n")


def create_mock_settings():
    """创建模拟设置"""
    settings = SettingManager()
    # 不再需要显式设置 PROXY_ENABLED，只要配置了 PROXY_API_URL 就会启用
    settings.set("PROXY_API_URL", "http://test.proxy.api/get")
    settings.set("LOG_LEVEL", "DEBUG")
    return settings


async def test_proxy_middleware_initialization():
    """测试代理中间件初始化"""
    print("=== 测试代理中间件初始化 ===")
    
    settings = create_mock_settings()
    middleware = ProxyMiddleware(settings, "DEBUG")
    
    print(f"代理中间件已启用: {middleware.enabled}")
    print(f"API URL: {middleware.api_url}")
    print(f"代理池大小: {middleware.proxy_pool_size}")
    print(f"健康检查阈值: {middleware.health_check_threshold}")
    print(f"刷新间隔: {middleware.refresh_interval}")
    
    print("代理中间件初始化测试完成\n")


async def test_proxy_pool_management():
    """测试代理池管理功能"""
    print("=== 测试代理池管理 ===")
    
    settings = create_mock_settings()
    middleware = ProxyMiddleware(settings, "DEBUG")
    
    # 模拟API响应
    mock_proxies = [
        "http://proxy1.example.com:8080",
        "http://proxy2.example.com:8080", 
        "http://proxy3.example.com:8080"
    ]
    
    # 测试更新代理池
    with patch.object(middleware, '_get_proxy_from_api', AsyncMock(return_value=mock_proxies[0])):
        await middleware._update_proxy_pool()
        print(f"代理池大小: {len(middleware._proxy_pool)}")
        if middleware._proxy_pool:
            print(f"第一个代理: {middleware._proxy_pool[0].proxy_str}")
    
    # 测试获取健康代理
    healthy_proxy = await middleware._get_healthy_proxy()
    if healthy_proxy:
        print(f"获取到健康代理: {healthy_proxy.proxy_str}")
    else:
        print("未获取到健康代理")
        
    print("代理池管理测试完成\n")


async def test_process_request():
    """测试请求处理"""
    print("=== 测试请求处理 ===")
    
    settings = create_mock_settings()
    middleware = ProxyMiddleware(settings, "DEBUG")
    
    # 创建模拟请求
    request = Request(url="http://example.com")
    
    # 创建模拟爬虫对象
    mock_spider = Mock()
    mock_spider.crawler.settings.get.return_value = "aiohttp"
    
    # 添加一些测试代理到池中
    middleware._proxy_pool = [
        Proxy("http://proxy1.example.com:8080"),
        Proxy("http://proxy2.example.com:8080")
    ]
    
    # 处理请求
    result = await middleware.process_request(request, mock_spider)
    print(f"处理结果: {result}")
    print(f"请求代理: {request.proxy}")
    if "_used_proxy" in request.meta:
        print(f"使用的代理对象: {request.meta['_used_proxy'].proxy_str}")
    
    print("请求处理测试完成\n")


def test_process_response():
    """测试响应处理"""
    print("=== 测试响应处理 ===")
    
    settings = create_mock_settings()
    middleware = ProxyMiddleware(settings, "DEBUG")
    
    # 创建带代理信息的请求
    request = Request(url="http://example.com")
    proxy_obj = Proxy("http://proxy1.example.com:8080")
    request.meta["_used_proxy"] = proxy_obj
    
    # 创建响应
    response = Response(
        url="http://example.com",
        status_code=200,
        body=b"test response",
        request=request
    )
    
    # 处理响应前
    print(f"处理前 - 代理成功次数: {proxy_obj.success_count}, 失败次数: {proxy_obj.failure_count}")
    
    # 处理响应
    result = middleware.process_response(request, response, None)
    
    # 处理后
    print(f"处理后 - 代理成功次数: {proxy_obj.success_count}, 失败次数: {proxy_obj.failure_count}")
    print(f"成功率: {proxy_obj.success_rate}")
    
    print("响应处理测试完成\n")


def test_process_exception():
    """测试异常处理"""
    print("=== 测试异常处理 ===")
    
    settings = create_mock_settings()
    middleware = ProxyMiddleware(settings, "DEBUG")
    
    # 创建带代理信息的请求
    request = Request(url="http://example.com")
    proxy_obj = Proxy("http://proxy1.example.com:8080")
    request.meta["_used_proxy"] = proxy_obj
    
    # 处理异常前
    print(f"处理前 - 代理成功次数: {proxy_obj.success_count}, 失败次数: {proxy_obj.failure_count}")
    
    # 处理异常
    result = middleware.process_exception(request, Exception("Test error"), None)
    
    # 处理后
    print(f"处理后 - 代理成功次数: {proxy_obj.success_count}, 失败次数: {proxy_obj.failure_count}")
    print(f"成功率: {proxy_obj.success_rate}")
    print(f"是否健康: {proxy_obj.is_healthy}")
    
    print("异常处理测试完成\n")


async def main():
    """主测试函数"""
    print("开始测试增强版代理中间件...\n")
    
    # 运行各个测试
    test_proxy_class()
    await test_proxy_middleware_initialization()
    await test_proxy_pool_management()
    await test_process_request()
    test_process_response()
    test_process_exception()
    
    print("所有测试完成!")


if __name__ == "__main__":
    asyncio.run(main())


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Redis Key 验证演示脚本
演示如何使用Redis Key验证工具
"""
import sys
import os

# 添加项目根目录到路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.utils.redis_manager import (
    RedisKeyValidator, 
    validate_redis_key_naming, 
    validate_multiple_redis_keys,
    get_redis_key_info,
    print_validation_report
)


def demonstrate_redis_key_validation():
    """演示Redis Key验证功能"""
    print("=== Redis Key 验证功能演示 ===\n")
    
    # 1. 验证单个有效的Redis Key
    print("1. 验证单个有效的Redis Key:")
    valid_keys = [
        "crawlo:books_distributed:filter:fingerprint",
        "crawlo:books_distributed:queue:requests",
        "crawlo:books_distributed:queue:processing",
        "crawlo:books_distributed:queue:failed",
        "crawlo:books_distributed:item:fingerprint"
    ]
    
    for key in valid_keys:
        is_valid = validate_redis_key_naming(key, "books_distributed")
        print(f"  {key} - {'通过' if is_valid else '失败'}")
    
    print()
    
    # 2. 验证无效的Redis Key
    print("2. 验证无效的Redis Key:")
    invalid_keys = [
        "invalid_format",  # 不以crawlo开头
        "crawlo:books_distributed",  # 部分缺失
        "crawlo:books_distributed:invalid_component:fingerprint",  # 无效组件
        "crawlo:books_distributed:queue:invalid_subcomponent",  # 无效子组件
        "crawlo:wrong_project:filter:fingerprint"  # 项目名称不匹配
    ]
    
    for key in invalid_keys:
        is_valid = validate_redis_key_naming(key, "books_distributed")
        print(f"  {key} - {'通过' if is_valid else '失败'}")
    
    print()
    
    # 3. 批量验证Redis Key
    print("3. 批量验证Redis Key:")
    all_keys = valid_keys + invalid_keys
    is_valid, invalid_keys_list = validate_multiple_redis_keys(all_keys, "books_distributed")
    
    print(f"  批量验证结果: {'全部通过' if is_valid else '存在无效Key'}")
    if not is_valid:
        print("  无效的Key:")
        for key in invalid_keys_list:
            print(f"    - {key}")
    
    print()
    
    # 4. 获取Redis Key信息
    print("4. 获取Redis Key信息:")
    sample_keys = [
        "crawlo:api_data_collection:filter:fingerprint",
        "crawlo:api_data_collection:queue:requests",
        "crawlo:api_data_collection:item:fingerprint"
    ]
    
    for key in sample_keys:
        info = get_redis_key_info(key)
        print(f"  Key: {key}")
        if info['valid']:
            print(f"    框架: {info['framework']}")
            print(f"    项目: {info['project']}")
            print(f"    组件: {info['component']}")
            if 'sub_component' in info:
                print(f"    子组件: {info['sub_component']}")
        else:
            print(f"    错误: {info.get('error', '无效')}")
        print()
    
    # 5. 打印验证报告
    print("5. 打印验证报告:")
    print_validation_report(all_keys, "books_distributed")


def demonstrate_advanced_validation():
    """演示高级验证功能"""
    print("\n=== 高级验证功能演示 ===\n")
    
    validator = RedisKeyValidator()
    
    # 1. 不指定项目名称的验证
    print("1. 不指定项目名称的验证:")
    key = "crawlo:any_project:filter:fingerprint"
    is_valid = validator.validate_key_naming(key)  # 不指定项目名称
    print(f"  {key} - {'通过' if is_valid else '失败'}")
    
    # 2. 验证不同项目的Key
    print("\n2. 验证不同项目的Key:")
    project_keys = {
        "books_distributed": [
            "crawlo:books_distributed:filter:fingerprint",
            "crawlo:books_distributed:queue:requests"
        ],
        "api_data_collection": [
            "crawlo:api_data_collection:filter:fingerprint",
            "crawlo:api_data_collection:queue:requests"
        ]
    }
    
    for project_name, keys in project_keys.items():
        print(f"  项目: {project_name}")
        for key in keys:
            is_valid = validator.validate_key_naming(key, project_name)
            print(f"    {key} - {'通过' if is_valid else '失败'}")


if __name__ == "__main__":
    demonstrate_redis_key_validation()
    demonstrate_advanced_validation()


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
测试请求优先级行为
验证优先级值越小越优先的规则
"""

import sys
import os
import asyncio
from unittest.mock import Mock, patch

# 添加项目根目录到路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.network.request import Request
from crawlo.queue.pqueue import SpiderPriorityQueue
from crawlo.queue.redis_priority_queue import RedisPriorityQueue


async def test_memory_queue_priority():
    """测试内存队列的优先级行为"""
    print("=== 测试内存队列优先级行为 ===")
    
    queue = SpiderPriorityQueue()
    
    # 创建不同优先级的请求
    request_low_priority = Request(url="https://low-priority.com", priority=100)   # 低优先级（数值大）
    request_high_priority = Request(url="https://high-priority.com", priority=-100)  # 高优先级（数值小）
    request_normal_priority = Request(url="https://normal-priority.com", priority=0)  # 正常优先级
    
    # 按照"数值小优先级高"的原则入队
    await queue.put((-100, request_high_priority))  # 高优先级先入队
    await queue.put((0, request_normal_priority))   # 正常优先级
    await queue.put((100, request_low_priority))    # 低优先级最后入队
    
    print(f"  队列大小: {queue.qsize()}")
    
    # 出队顺序应该按照优先级从高到低
    item1 = await queue.get(timeout=1.0)
    item2 = await queue.get(timeout=1.0)
    item3 = await queue.get(timeout=1.0)
    
    assert item1 is not None and item1[1].url == "https://high-priority.com", "高优先级应该先出队"
    assert item2 is not None and item2[1].url == "https://normal-priority.com", "正常优先级应该第二个出队"
    assert item3 is not None and item3[1].url == "https://low-priority.com", "低优先级应该最后出队"
    
    print("  ✅ 内存队列优先级测试通过")


async def test_redis_queue_priority():
    """测试Redis队列的优先级行为"""
    print("\n=== 测试Redis队列优先级行为 ===")
    
    # 使用测试专用的Redis数据库
    queue = RedisPriorityQueue(
        redis_url="redis://127.0.0.1:6379/15",
        queue_name="test:priority:queue"
    )
    
    try:
        await queue.connect()
        
        # 清理之前的测试数据
        await queue._redis.delete(queue.queue_name)
        await queue._redis.delete(f"{queue.queue_name}:data")
        
        # 创建不同优先级的请求
        # 注意：Request构造函数会将传入的priority值取反存储
        # 所以priority=100的请求实际存储为-100，priority=-100的请求实际存储为100
        request_low_priority = Request(url="https://low-priority.com", priority=100)   # 实际存储为-100（高优先级）
        request_high_priority = Request(url="https://high-priority.com", priority=-100)  # 实际存储为100（低优先级）
        request_normal_priority = Request(url="https://normal-priority.com", priority=0)  # 实际存储为0（正常优先级）
        
        # 按照正确的顺序入队以验证优先级行为
        # 使用实际存储的priority值
        await queue.put(request_low_priority, priority=request_low_priority.priority)    # 实际score=-100
        await queue.put(request_normal_priority, priority=request_normal_priority.priority)   # 实际score=0
        await queue.put(request_high_priority, priority=request_high_priority.priority)  # 实际score=100
        
        print(f"  队列大小: {await queue.qsize()}")
        
        # 出队顺序应该按照score从小到大（priority从小到大）
        # 所以request_low_priority先出队（score=-100），request_normal_priority第二个出队（score=0），request_high_priority最后出队（score=100）
        item1 = await queue.get(timeout=2.0)
        item2 = await queue.get(timeout=2.0)
        item3 = await queue.get(timeout=2.0)
        
        # 验证出队顺序
        print(f"  第一个出队: {item1.url if item1 else None}")
        print(f"  第二个出队: {item2.url if item2 else None}")
        print(f"  第三个出队: {item3.url if item3 else None}")
        
        # Redis队列中，score小的先出队，所以priority小的先出队
        assert item1 is not None and item1.url == "https://low-priority.com", f"低优先级请求应该先出队，实际: {item1.url if item1 else None}"
        assert item2 is not None and item2.url == "https://normal-priority.com", f"正常优先级请求应该第二个出队，实际: {item2.url if item2 else None}"
        assert item3 is not None and item3.url == "https://high-priority.com", f"高优先级请求应该最后出队，实际: {item3.url if item3 else None}"
        
        print("  ✅ Redis队列优先级测试通过（确认了score越小越优先的规则）")
        print("  注意：Redis队列中score = priority，所以priority值小的请求score小，会先出队")
        
    except Exception as e:
        print(f"  ❌ Redis队列优先级测试失败: {e}")
        import traceback
        traceback.print_exc()
    finally:
        await queue.close()


async def test_priority_values():
    """测试优先级常量值"""
    print("\n=== 测试优先级常量值 ===")
    
    from crawlo.network.request import RequestPriority
    
    # 检查优先级常量值
    print(f"  URGENT: {RequestPriority.URGENT}")
    print(f"  HIGH: {RequestPriority.HIGH}")
    print(f"  NORMAL: {RequestPriority.NORMAL}")
    print(f"  LOW: {RequestPriority.LOW}")
    print(f"  BACKGROUND: {RequestPriority.BACKGROUND}")
    
    # 验证数值越小优先级越高
    assert RequestPriority.URGENT < RequestPriority.HIGH < RequestPriority.NORMAL < RequestPriority.LOW < RequestPriority.BACKGROUND
    print("  ✅ 优先级常量值测试通过")


async def test_retry_middleware_priority():
    """测试重试中间件中的优先级调整"""
    print("\n=== 测试重试中间件优先级调整 ===")
    
    from crawlo.middleware.retry import RetryMiddleware
    from crawlo.stats_collector import StatsCollector
    from crawlo.settings.setting_manager import SettingManager
    
    # 创建设置管理器
    settings = SettingManager()
    settings.set('RETRY_HTTP_CODES', [500])
    settings.set('IGNORE_HTTP_CODES', [])
    settings.set('MAX_RETRY_TIMES', 3)
    settings.set('RETRY_EXCEPTIONS', [])
    settings.set('RETRY_PRIORITY', -1)  # 重试时降低优先级
    
    # 创建统计收集器
    class MockCrawler:
        def __init__(self):
            self.settings = settings
    
    crawler = MockCrawler()
    stats = StatsCollector(crawler)
    crawler.stats = stats
    
    class MockCrawlerWithStats:
        def __init__(self):
            self.settings = settings
            self.stats = stats
    
    crawler_with_stats = MockCrawlerWithStats()
    
    # 创建重试中间件
    middleware = RetryMiddleware.create_instance(crawler_with_stats)
    
    # 创建请求和爬虫（注意：这里设置优先级为-10，因为Request构造函数会将其转换为10）
    request = Request(url="https://example.com", priority=-10)  # 实际priority将为10
    spider = Mock()
    
    print(f"  原始请求优先级: {request.priority}")  # 应该是10
    
    # 模拟500错误响应
    class MockResponse:
        def __init__(self, status_code=200):
            self.status_code = status_code
    
    response = MockResponse(500)
    result = middleware.process_response(request, response, spider)
    
    # 应该返回重试的请求，优先级应该降低
    assert result is not None
    assert result is request  # 同一个对象
    assert result.priority == 9  # 原始优先级10 + RETRY_PRIORITY(-1) = 9
    print(f"  重试后请求优先级: {result.priority}")
    print("  ✅ 重试中间件优先级调整测试通过")


async def main():
    print("开始测试请求优先级行为...")
    
    try:
        # 运行所有测试
        await test_priority_values()
        await test_memory_queue_priority()
        await test_redis_queue_priority()
        await test_retry_middleware_priority()
        
        print("\n🎉 所有优先级行为测试通过！")
        print("\n总结:")
        print("1. 请求优先级遵循'数值越小越优先'的原则")
        print("2. 内存队列: 直接使用(priority, request)元组，priority小的先出队")
        print("3. Redis队列: 使用score = priority，score小的先出队，所以priority小的先出队")
        print("   现在内存队列和Redis队列行为一致")
        print("4. 重试中间件会根据RETRY_PRIORITY配置调整请求优先级")
        print("5. 系统内置的优先级常量: URGENT(-200) < HIGH(-100) < NORMAL(0) < LOW(100) < BACKGROUND(200)")
        print("6. Request对象构造时会将传入的priority值取反存储")
        
    except Exception as e:
        print(f"\n❌ 测试失败: {e}")
        import traceback
        traceback.print_exc()


if __name__ == "__main__":
    asyncio.run(main())


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
简单测试 QUEUE_TYPE 配置获取
验证我们的日志格式修改是否正常工作
"""

import sys
import os

# 添加项目根目录到路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.config import CrawloConfig
from crawlo.framework import CrawloFramework


def test_log_format():
    """测试日志格式修改是否正常工作"""
    print("=== 测试日志格式修改 ===")
    
    # 创建单机模式配置
    config = CrawloConfig.standalone(concurrency=4)
    
    # 创建框架实例，这会触发日志输出
    framework = CrawloFramework(config.to_dict())
    
    # 获取配置信息
    run_mode = framework.settings.get('RUN_MODE', 'not found')
    queue_type = framework.settings.get('QUEUE_TYPE', 'not found')
    
    print(f"从配置中获取到的信息:")
    print(f"  RunMode: {run_mode}")
    print(f"  QueueType: {queue_type}")
    
    print("\n✅ 日志格式修改测试完成")


if __name__ == "__main__":
    print("开始简单测试 QUEUE_TYPE 配置获取...")
    test_log_format()
    print("\n测试结束！")


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
性能监控工具测试
测试 PerformanceMonitor, PerformanceTimer, performance_monitor_decorator
"""
import sys
import os
import unittest
from unittest.mock import Mock, patch, MagicMock
import asyncio

# 添加项目根目录到 Python 路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

# 尝试导入性能监控工具
try:
    from crawlo.utils.performance_monitor import PerformanceMonitor, PerformanceTimer, performance_monitor_decorator
    PSUTIL_AVAILABLE = True
except ImportError:
    PSUTIL_AVAILABLE = False
    PerformanceMonitor = None
    PerformanceTimer = None
    performance_monitor_decorator = None


class TestPerformanceTimer(unittest.TestCase):
    """性能计时器测试"""

    @unittest.skipIf(not PSUTIL_AVAILABLE, "psutil not available")
    def setUp(self):
        """测试前准备"""
        self.timer = PerformanceTimer("test_timer")
        
    @unittest.skipIf(not PSUTIL_AVAILABLE, "psutil not available")
    def test_timer_initialization(self):
        """测试计时器初始化"""
        self.assertEqual(self.timer.name, "test_timer")
        self.assertIsNone(self.timer.start_time)
        self.assertIsNone(self.timer.end_time)
        
    @unittest.skipIf(not PSUTIL_AVAILABLE, "psutil not available")
    def test_timer_context_manager(self):
        """测试计时器上下文管理器"""
        with self.timer as t:
            # 在这里执行一些操作
            result = 1 + 1
            
        self.assertEqual(result, 2)
        # 验证计时器已启动和停止
        self.assertIsNotNone(t.start_time)
        self.assertIsNotNone(t.end_time)
        
    @unittest.skipIf(not PSUTIL_AVAILABLE, "psutil not available")
    def test_timer_start_stop(self):
        """测试计时器启动和停止"""
        self.timer.start()
        self.assertIsNotNone(self.timer.start_time)
        
        # 等待一小段时间
        import time
        time.sleep(0.01)
        
        elapsed = self.timer.stop()
        self.assertIsNotNone(self.timer.end_time)
        self.assertIsInstance(elapsed, float)
        self.assertGreater(elapsed, 0)


class TestPerformanceMonitor(unittest.TestCase):
    """性能监控器测试"""

    @unittest.skipIf(not PSUTIL_AVAILABLE, "psutil not available")
    def setUp(self):
        """测试前准备"""
        self.monitor = PerformanceMonitor("test_monitor")
        
    @unittest.skipIf(not PSUTIL_AVAILABLE, "psutil not available")
    def test_monitor_initialization(self):
        """测试监控器初始化"""
        self.assertEqual(self.monitor.start_time, self.monitor.start_time)
        self.assertIsInstance(self.monitor.metrics, dict)
        self.assertIn('cpu_usage', self.monitor.metrics)
        self.assertIn('memory_usage', self.monitor.metrics)
        self.assertIn('network_io', self.monitor.metrics)
        self.assertIn('disk_io', self.monitor.metrics)


class TestPerformanceMonitorDecorator(unittest.TestCase):
    """性能监控装饰器测试"""

    @unittest.skipIf(not PSUTIL_AVAILABLE, "psutil not available")
    def test_performance_monitor_decorator_sync(self):
        """测试同步函数的性能监控装饰器"""
        @performance_monitor_decorator(name="test_sync_function")
        def sync_function():
            return "test_result"
            
        result = sync_function()
        self.assertEqual(result, "test_result")
        
    @unittest.skipIf(not PSUTIL_AVAILABLE, "psutil not available")
    def test_performance_monitor_decorator_async(self):
        """测试异步函数的性能监控装饰器"""
        @performance_monitor_decorator(name="test_async_function")
        async def async_function():
            await asyncio.sleep(0.01)  # 模拟异步操作
            return "async_result"
            
        # 使用事件循环运行异步函数
        result = asyncio.run(async_function())
        self.assertEqual(result, "async_result")


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python
# -*- coding:UTF-8 -*-
import time
import asyncio
from typing import Set, Final
from collections import deque
from asyncio import Task, Future, Semaphore
from crawlo.logging import get_logger


class DynamicSemaphore(Semaphore):
    """支持动态调整的信号量"""
    
    def __init__(self, initial_value: int = 8):
        super().__init__(initial_value)
        self._initial_value = initial_value
        self._current_value = initial_value
        self._response_times = deque(maxlen=10)  # 存储最近10次响应时间
        self._last_adjust_time = time.time()
        
    def record_response_time(self, response_time: float):
        """记录响应时间"""
        self._response_times.append(response_time)
        
    def adjust_concurrency(self):
        """根据响应时间动态调整并发数"""
        current_time = time.time()
        # 限制调整频率，至少间隔1秒（从2秒减少到1秒）
        if current_time - self._last_adjust_time < 1:
            return
            
        self._last_adjust_time = current_time
        
        if len(self._response_times) < 2:  # 从3减少到2
            return
            
        # 计算平均响应时间
        avg_response_time = sum(self._response_times) / len(self._response_times)
        
        # 根据响应时间调整并发数
        if avg_response_time < 0.2:  # 响应很快，增加并发（从0.3降到0.2）
            new_concurrency = min(self._current_value + 5, self._initial_value * 3)  # 增加幅度从3提高到5，最大值从2倍提高到3倍
        elif avg_response_time > 1.0:  # 响应较慢，减少并发（从1.5降到1.0）
            new_concurrency = max(self._current_value - 5, max(1, self._initial_value // 3))  # 减少幅度从3提高到5，最小值从一半降低到三分之一
        else:
            return  # 保持当前并发数
            
        # 只有当变化较大时才调整
        if abs(new_concurrency - self._current_value) > 1:
            self._adjust_semaphore_value(new_concurrency)
    
    def _adjust_semaphore_value(self, new_value: int):
        """调整信号量的值"""
        if new_value > self._current_value:
            # 增加信号量
            for _ in range(new_value - self._current_value):
                self.release()
        elif new_value < self._current_value:
            # 减少信号量，这里只是记录新的目标值
            # 实际减少会在acquire时处理
            pass
            
        self._current_value = new_value
        # 注意：Python的Semaphore没有直接修改内部计数器的方法
        # 所以我们只能通过release()来增加，减少则需要在acquire时控制


class TaskManager:

    def __init__(self, total_concurrency: int = 8):
        self.current_task: Final[Set] = set()
        # 使用动态信号量替代普通信号量
        self.semaphore: DynamicSemaphore = DynamicSemaphore(max(1, total_concurrency))
        self.logger = get_logger(self.__class__.__name__)
        
        # 异常统计
        self._exception_count = 0
        self._total_tasks = 0

    async def create_task(self, coroutine) -> Task:
        # 等待信号量，控制并发数
        await self.semaphore.acquire()
        
        task = asyncio.create_task(coroutine)
        self.current_task.add(task)
        self._total_tasks += 1

        def done_callback(_future: Future) -> None:
            try:
                self.current_task.discard(task)  # 使用discard而不是remove，避免KeyError
                
                # 获取任务结果或异常 - 这是关键，必须调用result()或exception()来"获取"异常
                try:
                    # 尝试获取结果，如果有异常会被抛出
                    result = _future.result()
                    # 如果成功完成，可以在这里记录成功统计
                except asyncio.CancelledError:
                    # 正确处理取消异常，避免"never retrieved"警告
                    self.logger.info("Task was cancelled")
                except Exception as exception:
                    # 异常被正确"获取"了，不会再出现"never retrieved"警告
                    self._exception_count += 1
                    
                    # 记录异常详情
                    self.logger.error(
                        f"Task completed with exception: {type(exception).__name__}: {exception}"
                    )
                    self.logger.debug("Task exception details:", exc_info=exception)
                    
                    # 可以在这里添加更多的异常处理逻辑，如发送到监控系统
                    
            except Exception as e:
                # 防止回调函数本身出现异常
                self.logger.error(f"Error in task done callback: {e}")
            finally:
                # 确保信号量始终被释放
                self.semaphore.release()
                
                # 定期调整并发数（从每3个任务调整一次改为每2个任务调整一次）
                if self._total_tasks % 2 == 0:
                    self.semaphore.adjust_concurrency()

        task.add_done_callback(done_callback)

        return task

    def all_done(self) -> bool:
        return len(self.current_task) == 0
    
    def record_response_time(self, response_time: float):
        """记录任务的响应时间，用于动态调整并发数"""
        self.semaphore.record_response_time(response_time)
    
    def get_stats(self) -> dict:
        """获取任务管理器统计信息"""
        return {
            'active_tasks': len(self.current_task),
            'total_tasks': self._total_tasks,
            'exception_count': self._exception_count,
            'success_rate': (self._total_tasks - self._exception_count) / max(1, self._total_tasks) * 100,
            'current_concurrency': self.semaphore._current_value
        }


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
ResponseFilterMiddleware 测试脚本
验证ResponseFilterMiddleware的功能是否正常工作
"""

import asyncio
import sys
import os

# 添加项目根目录到Python路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.middleware.response_filter import ResponseFilterMiddleware
from crawlo.network.request import Request
from crawlo.network.response import Response
from crawlo.settings.setting_manager import SettingManager
from crawlo.exceptions import IgnoreRequestError


class MockStats:
    """模拟统计对象"""
    def __init__(self):
        self.counts = {}
        
    def inc_value(self, key, count=1):
        if key in self.counts:
            self.counts[key] += count
        else:
            self.counts[key] = count


class MockLogger:
    """模拟日志对象"""
    def __init__(self):
        self.messages = []
        
    def debug(self, message):
        self.messages.append(f"DEBUG: {message}")


class MockCrawler:
    """模拟爬虫实例"""
    def __init__(self, settings):
        self.settings = settings
        self.stats = MockStats()
        self.logger = MockLogger()


def create_test_settings(allowed_codes=None, denied_codes=None):
    """创建测试设置"""
    settings = SettingManager()
    settings.set("LOG_LEVEL", "DEBUG")
    
    if allowed_codes is not None:
        # 将列表转换为字符串格式存储
        settings.set("ALLOWED_RESPONSE_CODES", ",".join(map(str, allowed_codes)))
    else:
        settings.set("ALLOWED_RESPONSE_CODES", "")
        
    if denied_codes is not None:
        # 将列表转换为字符串格式存储
        settings.set("DENIED_RESPONSE_CODES", ",".join(map(str, denied_codes)))
    else:
        settings.set("DENIED_RESPONSE_CODES", "")
        
    return settings


async def test_response_filter_middleware():
    """测试ResponseFilterMiddleware功能"""
    print("开始测试ResponseFilterMiddleware...")
    
    # 测试用例
    test_cases = [
        # (状态码, 应该被允许, 描述)
        (200, True, "正常响应"),
        (201, True, "创建成功"),
        (299, True, "2xx范围内的其他状态码"),
        (404, False, "页面未找到"),
        (500, False, "服务器错误"),
        (403, False, "禁止访问"),
        (301, False, "重定向"),
    ]
    
    # 创建设置和中间件（不配置ALLOWED_RESPONSE_CODES，只允许2xx状态码）
    settings = create_test_settings(None, None)
    crawler = MockCrawler(settings)
    middleware = ResponseFilterMiddleware.create_instance(crawler)
    
    print(f"✓ ResponseFilterMiddleware创建成功")
    print(f"  默认允许的状态码: 200-299")
    print(f"  允许列表: {middleware.allowed_codes}")
    print(f"  拒绝列表: {middleware.denied_codes}")
    
    # 运行测试用例
    passed = 0
    total = len(test_cases)
    
    for status_code, should_allow, description in test_cases:
        try:
            request = Request(url="https://example.com")
            response = Response(
                url="https://example.com",
                status_code=status_code,
                body=b"Test response body"
            )
            
            result = middleware.process_response(request, response, None)
            
            # 如果到达这里，说明响应被允许
            if should_allow:
                if result == response:
                    print(f"✓ {description}: 状态码 {status_code} (正确允许)")
                    passed += 1
                else:
                    print(f"✗ {description}: 状态码 {status_code} (应该被允许但返回结果不正确)")
            else:
                print(f"✗ {description}: 状态码 {status_code} (应该被过滤但被允许)")
                
        except IgnoreRequestError as e:
            # 响应被过滤
            if not should_allow:
                print(f"✓ {description}: 状态码 {status_code} (正确过滤)")
                passed += 1
            else:
                print(f"✗ {description}: 状态码 {status_code} (应该被允许但被过滤)")
        except Exception as e:
            print(f"✗ {description}: 状态码 {status_code} (出现异常: {e})")
    
    # 输出测试结果
    print(f"\n测试结果: {passed}/{total} 通过")
    if passed == total:
        print("✓ 所有测试通过，ResponseFilterMiddleware工作正常")
        return True
    else:
        print("✗ 部分测试失败，ResponseFilterMiddleware存在问题")
        return False


async def test_allowed_codes_configuration():
    """测试ALLOWED_RESPONSE_CODES配置功能"""
    print("\n测试ALLOWED_RESPONSE_CODES配置功能...")
    
    # 配置允许的状态码
    allowed_codes = [404, 500, 301]
    settings = create_test_settings(allowed_codes, None)
    crawler = MockCrawler(settings)
    middleware = ResponseFilterMiddleware.create_instance(crawler)
    
    print(f"✓ ResponseFilterMiddleware创建成功")
    print(f"  配置的允许状态码: {allowed_codes}")
    print(f"  实际允许列表: {middleware.allowed_codes}")
    
    # 测试用例
    test_cases = [
        # (状态码, 应该被允许, 描述)
        (200, True, "正常响应(2xx范围)"),
        (404, True, "页面未找到(配置允许)"),
        (500, True, "服务器错误(配置允许)"),
        (301, True, "重定向(配置允许)"),
        (403, False, "禁止访问(未配置允许)"),
        (502, False, "网关错误(未配置允许)"),
    ]
    
    passed = 0
    total = len(test_cases)
    
    for status_code, should_allow, description in test_cases:
        try:
            request = Request(url="https://example.com")
            response = Response(
                url="https://example.com",
                status_code=status_code,
                body=b"Test response body"
            )
            
            result = middleware.process_response(request, response, None)
            
            # 如果到达这里，说明响应被允许
            if should_allow:
                if result == response:
                    print(f"✓ {description}: 状态码 {status_code} (正确允许)")
                    passed += 1
                else:
                    print(f"✗ {description}: 状态码 {status_code} (应该被允许但返回结果不正确)")
            else:
                print(f"✗ {description}: 状态码 {status_code} (应该被过滤但被允许)")
                
        except IgnoreRequestError as e:
            # 响应被过滤
            if not should_allow:
                print(f"✓ {description}: 状态码 {status_code} (正确过滤)")
                passed += 1
            else:
                print(f"✗ {description}: 状态码 {status_code} (应该被允许但被过滤)")
        except Exception as e:
            print(f"✗ {description}: 状态码 {status_code} (出现异常: {e})")
    
    # 输出测试结果
    print(f"\n测试结果: {passed}/{total} 通过")
    if passed == total:
        print("✓ 所有测试通过，ALLOWED_RESPONSE_CODES配置功能正常")
        return True
    else:
        print("✗ 部分测试失败，ALLOWED_RESPONSE_CODES配置功能存在问题")
        return False


async def test_denied_codes_configuration():
    """测试DENIED_RESPONSE_CODES配置功能"""
    print("\n测试DENIED_RESPONSE_CODES配置功能...")
    
    # 配置拒绝的状态码（注意：这里需要将整数转换为字符串，因为SettingManager.get_list会处理字符串）
    denied_codes = ["200", "201"]
    settings = create_test_settings(None, denied_codes)
    crawler = MockCrawler(settings)
    middleware = ResponseFilterMiddleware.create_instance(crawler)
    
    print(f"✓ ResponseFilterMiddleware创建成功")
    print(f"  配置的拒绝状态码: {denied_codes}")
    print(f"  实际拒绝列表: {middleware.denied_codes}")
    
    # 测试用例
    test_cases = [
        # (状态码, 应该被允许, 描述)
        (200, False, "正常响应(配置拒绝)"),
        (201, False, "创建成功(配置拒绝)"),
        (404, False, "页面未找到(默认拒绝)"),
        (202, True, "接受请求(2xx范围，未被拒绝)"),
        (301, False, "重定向(默认拒绝)"),
        (500, False, "服务器错误(默认拒绝)"),
    ]
    
    passed = 0
    total = len(test_cases)
    
    for status_code, should_allow, description in test_cases:
        try:
            request = Request(url="https://example.com")
            response = Response(
                url="https://example.com",
                status_code=status_code,
                body=b"Test response body"
            )
            
            result = middleware.process_response(request, response, None)
            
            # 如果到达这里，说明响应被允许
            if should_allow:
                if result == response:
                    print(f"✓ {description}: 状态码 {status_code} (正确允许)")
                    passed += 1
                else:
                    print(f"✗ {description}: 状态码 {status_code} (应该被允许但返回结果不正确)")
            else:
                print(f"✗ {description}: 状态码 {status_code} (应该被过滤但被允许)")
                
        except IgnoreRequestError as e:
            # 响应被过滤
            if not should_allow:
                print(f"✓ {description}: 状态码 {status_code} (正确过滤)")
                passed += 1
            else:
                print(f"✗ {description}: 状态码 {status_code} (应该被允许但被过滤)")
        except Exception as e:
            print(f"✗ {description}: 状态码 {status_code} (出现异常: {e})")
    
    # 输出测试结果
    print(f"\n测试结果: {passed}/{total} 通过")
    if passed == total:
        print("✓ 所有测试通过，DENIED_RESPONSE_CODES配置功能正常")
        return True
    else:
        print("✗ 部分测试失败，DENIED_RESPONSE_CODES配置功能存在问题")
        return False


def test_middleware_creation():
    """测试中间件创建功能"""
    print("\n测试中间件创建功能...")
    
    try:
        settings = create_test_settings(None, None)
        crawler = MockCrawler(settings)
        middleware = ResponseFilterMiddleware.create_instance(crawler)
        
        if middleware and hasattr(middleware, 'allowed_codes') and hasattr(middleware, 'denied_codes') and hasattr(middleware, 'logger'):
            print("✓ 中间件创建成功，属性完整")
            print(f"  allowed_codes: {middleware.allowed_codes}")
            print(f"  denied_codes: {middleware.denied_codes}")
            return True
        else:
            print("✗ 中间件创建失败，缺少必要属性")
            return False
    except Exception as e:
        print(f"✗ 中间件创建时出现异常: {e}")
        return False


async def test_filter_reason_function():
    """测试过滤原因功能"""
    print("\n测试过滤原因功能...")
    
    try:
        # 配置允许和拒绝的状态码（注意：这里需要将整数转换为字符串）
        allowed_codes = ["404"]
        denied_codes = ["200"]
        settings = create_test_settings(allowed_codes, denied_codes)
        crawler = MockCrawler(settings)
        middleware = ResponseFilterMiddleware.create_instance(crawler)
        
        # 测试拒绝状态码的原因
        reason1 = middleware._get_filter_reason(200)
        if "被明确拒绝" in reason1:
            print("✓ 拒绝状态码原因正确")
        else:
            print(f"✗ 拒绝状态码原因错误: {reason1}")
            
        # 测试未允许状态码的原因
        reason2 = middleware._get_filter_reason(500)
        if "不在允许列表中" in reason2:
            print("✓ 未允许状态码原因正确")
        else:
            print(f"✗ 未允许状态码原因错误: {reason2}")
            
        print("✓ 过滤原因功能正常")
        return True
        
    except Exception as e:
        print(f"✗ 过滤原因测试时出现异常: {e}")
        return False


async def test_is_response_allowed_function():
    """测试响应允许检查功能"""
    print("\n测试响应允许检查功能...")
    
    try:
        # 配置允许和拒绝的状态码（注意：这里需要将整数转换为字符串）
        allowed_codes = ["404", "301"]
        denied_codes = ["200"]
        settings = create_test_settings(allowed_codes, denied_codes)
        crawler = MockCrawler(settings)
        middleware = ResponseFilterMiddleware.create_instance(crawler)
        
        # 创建模拟响应对象
        class MockResponse:
            def __init__(self, status_code):
                self.status_code = status_code
        
        # 测试用例: (状态码, 应该被允许, 描述)
        test_cases = [
            (200, False, "被明确拒绝的状态码"),
            (404, True, "被明确允许的状态码"),
            (301, True, "被明确允许的状态码"),
            (201, True, "2xx范围内的默认允许状态码"),
            (500, False, "未被允许的错误状态码"),
        ]
        
        passed = 0
        total = len(test_cases)
        
        for status_code, should_allow, description in test_cases:
            response = MockResponse(status_code)
            is_allowed = middleware._is_response_allowed(response)
            
            if is_allowed == should_allow:
                print(f"✓ {description}: 状态码 {status_code}")
                passed += 1
            else:
                print(f"✗ {description}: 状态码 {status_code} (期望: {should_allow}, 实际: {is_allowed})")
        
        print(f"\n允许检查测试结果: {passed}/{total} 通过")
        if passed == total:
            print("✓ 响应允许检查功能正常")
            return True
        else:
            print("✗ 响应允许检查功能存在问题")
            return False
        
    except Exception as e:
        print(f"✗ 响应允许检查测试时出现异常: {e}")
        return False


async def main():
    """主测试函数"""
    print("ResponseFilterMiddleware 功能测试")
    print("=" * 50)
    
    # 测试正常功能
    test1_result = await test_response_filter_middleware()
    
    # 测试ALLOWED_RESPONSE_CODES配置
    test2_result = await test_allowed_codes_configuration()
    
    # 测试DENIED_RESPONSE_CODES配置
    test3_result = await test_denied_codes_configuration()
    
    # 测试创建功能
    test4_result = test_middleware_creation()
    
    # 测试过滤原因功能
    test5_result = await test_filter_reason_function()
    
    # 测试响应允许检查功能
    test6_result = await test_is_response_allowed_function()
    
    # 汇总结果
    print("\n" + "=" * 50)
    print("测试结果汇总:")
    print(f"基本功能测试: {'✓ 通过' if test1_result else '✗ 失败'}")
    print(f"允许配置测试: {'✓ 通过' if test2_result else '✗ 失败'}")
    print(f"拒绝配置测试: {'✓ 通过' if test3_result else '✗ 失败'}")
    print(f"创建测试: {'✓ 通过' if test4_result else '✗ 失败'}")
    print(f"原因功能测试: {'✓ 通过' if test5_result else '✗ 失败'}")
    print(f"允许检查测试: {'✓ 通过' if test6_result else '✗ 失败'}")
    
    overall_result = test1_result and test2_result and test3_result and test4_result and test5_result and test6_result
    print(f"\n总体结果: {'✓ 所有测试通过' if overall_result else '✗ 部分测试失败'}")
    
    return overall_result


if __name__ == "__main__":
    asyncio.run(main())


#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
ResponseFilterMiddleware 中间件
用于过滤不符合要求的HTTP响应，支持自定义允许的状态码
"""
from crawlo.logging import get_logger
from crawlo.exceptions import IgnoreRequestError


class ResponseFilterMiddleware:
    """
    ResponseFilterMiddleware 中间件
    用于过滤不符合要求的HTTP响应，支持自定义允许的状态码
    
    功能特性:
    - 默认允许2xx状态码
    - 支持自定义允许的状态码列表
    - 支持拒绝特定状态码
    - 提供详细的日志信息
    - 支持按域名配置不同的过滤规则
    """

    def __init__(self, allowed_codes, denied_codes):
        """
        初始化中间件
        
        Args:
            allowed_codes: 允许的状态码列表
            denied_codes: 拒绝的状态码列表
        """
        # 确保状态码是整数类型
        self.allowed_codes = set()
        if allowed_codes:
            for code in allowed_codes:
                try:
                    self.allowed_codes.add(int(code))
                except (ValueError, TypeError):
                    pass  # 忽略无效的状态码
                    
        self.denied_codes = set()
        if denied_codes:
            for code in denied_codes:
                try:
                    self.denied_codes.add(int(code))
                except (ValueError, TypeError):
                    pass  # 忽略无效的状态码
                    
        self.logger = get_logger(self.__class__.__name__)

    @classmethod
    def create_instance(cls, crawler):
        """
        创建中间件实例
        
        Args:
            crawler: 爬虫实例
            
        Returns:
            ResponseFilterMiddleware: 中间件实例
        """
        o = cls(
            allowed_codes=crawler.settings.get_list('ALLOWED_RESPONSE_CODES'),
            denied_codes=crawler.settings.get_list('DENIED_RESPONSE_CODES')
        )
        return o

    def _is_response_allowed(self, response):
        """
        判断响应是否被允许
        
        Args:
            response: 响应对象
            
        Returns:
            bool: 是否被允许
        """
        status_code = response.status_code
        
        # 首先检查是否被明确拒绝
        if status_code in self.denied_codes:
            return False
            
        # 检查是否被明确允许
        if status_code in self.allowed_codes:
            return True
            
        # 默认允许2xx状态码
        if 200 <= status_code < 300:
            return True
            
        # 默认拒绝其他状态码
        return False

    def _get_filter_reason(self, status_code):
        """
        获取过滤原因描述
        
        Args:
            status_code (int): HTTP状态码
            
        Returns:
            str: 过滤原因描述
        """
        if status_code in self.denied_codes:
            return f"状态码 {status_code} 被明确拒绝"
        elif status_code not in self.allowed_codes and not (200 <= status_code < 300):
            return f"状态码 {status_code} 不在允许列表中"
        else:
            return f"状态码 {status_code} 被过滤"

    def process_response(self, request, response, spider):
        """
        处理响应，过滤不符合要求的响应
        
        Args:
            request: 请求对象
            response: 响应对象
            spider: 爬虫实例
            
        Returns:
            response: 响应对象（如果被允许）
            
        Raises:
            IgnoreRequestError: 如果响应被过滤
        """
        if self._is_response_allowed(response):
            return response
            
        # 响应被过滤
        reason = self._get_filter_reason(response.status_code)
        self.logger.debug(f"过滤响应: {response.status_code} {response.url} - {reason}")
        
        # 抛出异常以忽略该响应
        raise IgnoreRequestError(f"response filtered: {reason} - {response.status_code} {response.url}")


#!/usr/bin/python
# -*- coding:UTF-8 -*-
import asyncio
from typing import List

try:
    from anyio import EndOfStream
except ImportError:
    # 如果 anyio 不可用或者 EndOfStream 不存在，创建一个占位符
    class EndOfStream(Exception):
        pass

try:
    from httpcore import ReadError
except ImportError:
    class ReadError(Exception):
        pass

try:
    from httpx import RemoteProtocolError, ConnectError, ReadTimeout
except ImportError:
    class RemoteProtocolError(Exception):
        pass
    class ConnectError(Exception):
        pass
    class ReadTimeout(Exception):
        pass

try:
    from aiohttp.client_exceptions import ClientConnectionError, ClientPayloadError
    from aiohttp import ClientConnectorError, ClientTimeout, ClientConnectorSSLError, ClientResponseError
except ImportError:
    class ClientConnectionError(Exception):
        pass
    class ClientPayloadError(Exception):
        pass
    class ClientConnectorError(Exception):
        pass
    class ClientTimeout(Exception):
        pass
    class ClientConnectorSSLError(Exception):
        pass
    class ClientResponseError(Exception):
        pass

from crawlo.logging import get_logger
from crawlo.stats_collector import StatsCollector

_retry_exceptions = [
    EndOfStream,
    ReadError,
    asyncio.TimeoutError,
    ConnectError,
    ReadTimeout,
    ClientConnectorError,
    ClientResponseError,
    RemoteProtocolError,
    ClientTimeout,
    ClientConnectorSSLError,
    ClientPayloadError,
    ClientConnectionError
]


class RetryMiddleware(object):

    def __init__(
            self,
            *,
            retry_http_codes: List,
            ignore_http_codes: List,
            max_retry_times: int,
            retry_exceptions: List,
            stats: StatsCollector,
            retry_priority: int
    ):
        self.retry_http_codes = retry_http_codes
        self.ignore_http_codes = ignore_http_codes
        self.max_retry_times = max_retry_times
        self.retry_exceptions = tuple(retry_exceptions + _retry_exceptions)
        self.retry_priority = retry_priority
        self.stats = stats
        self.logger = get_logger(self.__class__.__name__)

    @classmethod
    def create_instance(cls, crawler):
        o = cls(
            retry_http_codes=crawler.settings.get_list('RETRY_HTTP_CODES'),
            ignore_http_codes=crawler.settings.get_list('IGNORE_HTTP_CODES'),
            max_retry_times=crawler.settings.get_int('MAX_RETRY_TIMES'),
            retry_exceptions=crawler.settings.get_list('RETRY_EXCEPTIONS'),
            stats=crawler.stats,
            retry_priority=crawler.settings.get_int('RETRY_PRIORITY')
        )
        return o

    def process_response(self, request, response, spider):
        if request.meta.get('dont_retry', False):
            return response
        if response.status_code in self.ignore_http_codes:
            return response
        if response.status_code in self.retry_http_codes:
            # 重试逻辑
            reason = f"response code {response.status_code}"
            return self._retry(request, reason, spider) or response
        return response

    def process_exception(self, request, exc, spider):
        if isinstance(exc, self.retry_exceptions) and not request.meta.get('dont_retry', False):
            return self._retry(request=request, reason=type(exc).__name__, spider=spider)

    def _retry(self, request, reason, spider):
        retry_times = request.meta.get('retry_times', 0)
        if retry_times < self.max_retry_times:
            retry_times += 1
            self.logger.info(f"{spider} {request} {reason} retrying {retry_times} time...")
            request.meta['retry_times'] = retry_times
            # request.dont_retry = True
            request.meta['dont_retry'] = True
            request.priority = request.priority + self.retry_priority
            self.stats.inc_value("retry_count")
            # 添加重试标识，用于统计时识别
            request.meta['is_retry'] = True
            return request
        else:
            self.logger.warning(f"{spider} {request} {reason} retry max {self.max_retry_times} times, give up.")
            return None


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
队列阻塞行为测试脚本
验证队列满时的阻塞行为
"""

import sys
import os
import asyncio
import time
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.network.request import Request
from crawlo.queue.queue_manager import QueueConfig, QueueManager


async def test_queue_blocking_behavior():
    """测试队列阻塞行为"""
    print("开始测试队列阻塞行为...")
    
    # 初始化框架
    from crawlo.initialization import initialize_framework
    settings = initialize_framework()
    
    # 创建一个小型队列配置进行测试
    settings.set('SCHEDULER_MAX_QUEUE_SIZE', 5)  # 设置非常小的队列大小
    
    # 创建队列配置
    queue_config = QueueConfig.from_settings(settings)
    queue_config.max_queue_size = 5  # 设置小队列大小进行测试
    
    # 创建队列管理器
    queue_manager = QueueManager(queue_config)
    
    # 初始化队列
    await queue_manager.initialize()
    
    print(f"队列类型: {queue_manager._queue_type}")
    print(f"队列最大大小: {queue_config.max_queue_size}")
    
    # 创建生产者任务
    async def producer(queue_manager, name, count):
        """生产者：向队列添加请求"""
        print(f"生产者 {name} 开始工作，将添加 {count} 个请求")
        start_time = time.time()
        
        for i in range(count):
            request = Request(url=f'https://example.com/page{name}_{i}.html')
            try:
                # 这里应该会阻塞当队列满时
                await queue_manager.put(request)
                print(f"生产者 {name} 成功添加请求 {i}")
            except Exception as e:
                print(f"生产者 {name} 添加请求 {i} 失败: {e}")
        
        end_time = time.time()
        print(f"生产者 {name} 完成工作，耗时 {end_time - start_time:.2f} 秒")
    
    # 创建消费者任务
    async def consumer(queue_manager, name, count):
        """消费者：从队列获取请求"""
        print(f"消费者 {name} 开始工作，将获取 {count} 个请求")
        start_time = time.time()
        
        retrieved_count = 0
        while retrieved_count < count:
            try:
                request = await queue_manager.get(timeout=2.0)
                if request:
                    print(f"消费者 {name} 成功获取请求: {request.url}")
                    retrieved_count += 1
                    # 模拟处理时间
                    await asyncio.sleep(0.1)
                else:
                    print(f"消费者 {name} 超时，没有获取到请求")
                    break
            except Exception as e:
                print(f"消费者 {name} 获取请求失败: {e}")
                break
        
        end_time = time.time()
        print(f"消费者 {name} 完成工作，获取了 {retrieved_count} 个请求，耗时 {end_time - start_time:.2f} 秒")
    
    # 同时运行生产者和消费者
    print("开始并发测试...")
    
    # 生产者尝试添加10个请求到大小为5的队列
    # 消费者会逐渐消费，生产者应该会被阻塞直到有空间
    tasks = [
        asyncio.create_task(producer(queue_manager, "P1", 10)),
        asyncio.create_task(consumer(queue_manager, "C1", 10))
    ]
    
    # 等待所有任务完成
    await asyncio.gather(*tasks, return_exceptions=True)
    
    final_queue_size = await queue_manager.size()
    print(f"队列最终大小: {final_queue_size}")
    
    # 关闭队列
    await queue_manager.close()
    
    print("队列阻塞行为测试完成！")


def main():
    """主函数"""
    print("开始队列阻塞行为测试...")
    asyncio.run(test_queue_blocking_behavior())


if __name__ == "__main__":
    main()


#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
错误处理工具
提供详细、一致的错误处理和日志记录机制
"""
import traceback
from functools import wraps
from datetime import datetime
from typing import Optional, Callable, Any, Dict, List

from crawlo.logging import get_logger


class ErrorContext:
    """错误上下文信息"""
    
    def __init__(self, context: str = "", module: str = "", function: str = ""):
        self.context = context
        self.module = module
        self.function = function
        self.timestamp = datetime.now()
        
    def __str__(self):
        parts = []
        if self.module:
            parts.append(f"Module: {self.module}")
        if self.function:
            parts.append(f"Function: {self.function}")
        if self.context:
            parts.append(f"Context: {self.context}")
        parts.append(f"Time: {self.timestamp.strftime('%Y-%m-%d %H:%M:%S')}")
        return " | ".join(parts)


class DetailedException(Exception):
    """带有详细信息的异常基类"""
    
    def __init__(self, message: str, context: Optional[ErrorContext] = None, 
                 error_code: Optional[str] = None, **kwargs):
        super().__init__(message)
        self.context = context
        self.error_code = error_code
        self.details = kwargs
        self.timestamp = datetime.now()
        
    def __str__(self):
        base_msg = super().__str__()
        if self.context:
            return f"{base_msg} ({self.context})"
        return base_msg
    
    def get_full_details(self) -> Dict:
        """获取完整的错误详情"""
        return {
            "message": str(self),
            "error_code": self.error_code,
            "context": str(self.context) if self.context else None,
            "details": self.details,
            "timestamp": self.timestamp.isoformat(),
            "exception_type": self.__class__.__name__
        }


class EnhancedErrorHandler:
    """增强版错误处理器"""
    
    def __init__(self, logger_name: str = __name__, log_level: str = 'ERROR'):
        self.logger = get_logger(logger_name)
        self.error_history: List[Dict] = []  # 错误历史记录
        self.max_history_size = 100  # 最大历史记录数
    
    def handle_error(self, exception: Exception, context: Optional[ErrorContext] = None, 
                     raise_error: bool = True, log_error: bool = True,
                     extra_info: Optional[Dict] = None) -> Dict:
        """
        增强版错误处理
        
        Args:
            exception: 异常对象
            context: 错误上下文信息
            raise_error: 是否重新抛出异常
            log_error: 是否记录错误日志
            extra_info: 额外的错误信息
            
        Returns:
            包含错误详情的字典
        """
        # 构建错误详情
        error_details = {
            "exception": exception,
            "exception_type": type(exception).__name__,
            "message": str(exception),
            "context": str(context) if context else None,
            "timestamp": datetime.now().isoformat(),
            "traceback": traceback.format_exc() if log_error else None,
            "extra_info": extra_info or {}
        }
        
        # 记录到历史
        self._record_error(error_details)
        
        # 记录日志
        if log_error:
            self._log_error(error_details)
        
        # 重新抛出异常
        if raise_error:
            raise exception
        
        return error_details
    
    def _log_error(self, error_details: Dict):
        """记录错误日志"""
        # 基本错误信息
        context_info = error_details.get("context", "")
        message = error_details["message"]
        error_msg = f"{message} [{context_info}]" if context_info else message
        
        # 记录错误
        self.logger.error(error_msg)
        
        # 记录详细信息
        if error_details.get("traceback"):
            self.logger.debug(f"详细错误信息:\n{error_details['traceback']}")
        
        # 记录额外信息
        if error_details.get("extra_info"):
            self.logger.debug(f"额外信息: {error_details['extra_info']}")
    
    def _record_error(self, error_details: Dict):
        """记录错误到历史"""
        self.error_history.append(error_details)
        # 限制历史记录大小
        if len(self.error_history) > self.max_history_size:
            self.error_history.pop(0)
    
    def safe_call(self, func: Callable, *args, default_return=None, 
                  context: Optional[ErrorContext] = None, **kwargs) -> Any:
        """
        安全调用函数，捕获并处理异常
        
        Args:
            func: 要调用的函数
            *args: 函数参数
            default_return: 默认返回值
            context: 错误上下文
            **kwargs: 函数关键字参数
            
        Returns:
            函数返回值或默认值
        """
        try:
            return func(*args, **kwargs)
        except Exception as e:
            self.handle_error(e, context=context, raise_error=False)
            return default_return
    
    def retry_on_failure(self, max_retries: int = 3, delay: float = 1.0, 
                         exceptions: tuple = (Exception,), backoff_factor: float = 1.0,
                         context: Optional[ErrorContext] = None):
        """
        装饰器：失败时重试（增强版）
        
        Args:
            max_retries: 最大重试次数
            delay: 初始重试间隔（秒）
            exceptions: 需要重试的异常类型
            backoff_factor: 退避因子（每次重试间隔乘以此因子）
            context: 错误上下文
        """
        def decorator(func):
            @wraps(func)
            async def async_wrapper(*args, **kwargs):
                last_exception = None
                current_delay = delay
                
                for attempt in range(max_retries + 1):
                    try:
                        return await func(*args, **kwargs)
                    except exceptions as e:
                        last_exception = e
                        if attempt < max_retries:
                            # 记录重试信息
                            retry_context = ErrorContext(
                                context=f"函数 {func.__name__} 执行失败 (尝试 {attempt + 1}/{max_retries + 1})",
                                module=context.module if context else "",
                                function=func.__name__
                            ) if context else None
                            
                            self.logger.warning(
                                f"函数 {func.__name__} 执行失败 (尝试 {attempt + 1}/{max_retries + 1}): {e}"
                            )
                            
                            import asyncio
                            await asyncio.sleep(current_delay)
                            current_delay *= backoff_factor  # 指数退避
                        else:
                            # 最后一次尝试失败
                            final_context = ErrorContext(
                                context=f"函数 {func.__name__} 执行失败，已达到最大重试次数",
                                module=context.module if context else "",
                                function=func.__name__
                            ) if context else None
                            
                            self.logger.error(
                                f"函数 {func.__name__} 执行失败，已达到最大重试次数: {e}"
                            )
                raise last_exception
            
            @wraps(func)
            def sync_wrapper(*args, **kwargs):
                last_exception = None
                current_delay = delay
                
                for attempt in range(max_retries + 1):
                    try:
                        return func(*args, **kwargs)
                    except exceptions as e:
                        last_exception = e
                        if attempt < max_retries:
                            # 记录重试信息
                            retry_context = ErrorContext(
                                context=f"函数 {func.__name__} 执行失败 (尝试 {attempt + 1}/{max_retries + 1})",
                                module=context.module if context else "",
                                function=func.__name__
                            ) if context else None
            
                            self.logger.warning(
                                f"函数 {func.__name__} 执行失败 (尝试 {attempt + 1}/{max_retries + 1}): {e}"
                            )
                            
                            import time
                            time.sleep(current_delay)
                            current_delay *= backoff_factor  # 指数退避
                        else:
                            # 最后一次尝试失败
                            final_context = ErrorContext(
                                context=f"函数 {func.__name__} 执行失败，已达到最大重试次数",
                                module=context.module if context else "",
                                function=func.__name__
                            ) if context else None
                            
                            self.logger.error(
                                f"函数 {func.__name__} 执行失败，已达到最大重试次数: {e}"
                            )
                raise last_exception
            
            # 根据函数是否为异步函数返回相应的包装器
            import inspect
            if inspect.iscoroutinefunction(func):
                return async_wrapper
            else:
                return sync_wrapper
        
        return decorator
    
    def get_error_history(self) -> List[Dict]:
        """获取错误历史记录"""
        return self.error_history.copy()
    
    def clear_error_history(self):
        """清空错误历史记录"""
        self.error_history.clear()


# 为了向后兼容，提供与旧版error_handler.py相同的接口

# 别名，保持与旧版接口一致
ErrorHandler = EnhancedErrorHandler

# 全局增强错误处理器实例
enhanced_error_handler = EnhancedErrorHandler()

# 为了向后兼容，提供默认错误处理器实例
default_error_handler = enhanced_error_handler

def handle_exception(context: str = "", module: str = "", function: str = "",
                     raise_error: bool = True, log_error: bool = True,
                     error_code: Optional[str] = None):
    """
    装饰器：处理函数异常（增强版）
    
    Args:
        context: 错误上下文描述
        module: 模块名称
        function: 函数名称
        raise_error: 是否重新抛出异常
        log_error: 是否记录错误日志
        error_code: 错误代码
    """
    def decorator(func):
        @wraps(func)
        async def async_wrapper(*args, **kwargs):
            try:
                return await func(*args, **kwargs)
            except Exception as e:
                error_context = ErrorContext(
                    context=f"{context} - {func.__name__}",
                    module=module,
                    function=func.__name__
                )
                
                # 如果是详细异常，保留原有信息
                if isinstance(e, DetailedException):
                    # 确保上下文信息完整
                    if not e.context:
                        e.context = error_context
                    enhanced_error_handler.handle_error(
                        e, context=e.context,
                        raise_error=raise_error, log_error=log_error
                    )
                else:
                    # 包装为详细异常
                    detailed_e = DetailedException(
                        str(e), context=error_context, error_code=error_code
                    )
                    enhanced_error_handler.handle_error(
                        detailed_e, context=error_context,
                        raise_error=raise_error, log_error=log_error
                    )
                if not raise_error:
                    return None
        
        @wraps(func)
        def sync_wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                error_context = ErrorContext(
                    context=f"{context} - {func.__name__}",
                    module=module,
                    function=func.__name__
                )
                
                # 如果是详细异常，保留原有信息
                if isinstance(e, DetailedException):
                    # 确保上下文信息完整
                    if not e.context:
                        e.context = error_context
                    enhanced_error_handler.handle_error(
                        e, context=e.context,
                        raise_error=raise_error, log_error=log_error
                    )
                else:
                    # 包装为详细异常
                    detailed_e = DetailedException(
                        str(e), context=error_context, error_code=error_code
                    )
                    enhanced_error_handler.handle_error(
                        detailed_e, context=error_context,
                        raise_error=raise_error, log_error=log_error
                    )
                if not raise_error:
                    return None
        
        # 根据函数是否为异步函数返回相应的包装器
        import inspect
        if inspect.iscoroutinefunction(func):
            return async_wrapper
        else:
            return sync_wrapper
    
    return decorator

# 为了完全向后兼容，也提供旧版的handle_exception接口
# 这个版本与error_handler.py中的接口完全一致
def handle_exception_simple(context: str = "", raise_error: bool = True, log_error: bool = True):
    """
    装饰器：处理函数异常（简化版，与旧版接口兼容）
    
    Args:
        context: 错误上下文描述
        raise_error: 是否重新抛出异常
        log_error: 是否记录错误日志
    """
    def decorator(func):
        @wraps(func)
        async def async_wrapper(*args, **kwargs):
            try:
                return await func(*args, **kwargs)
            except Exception as e:
                error_context = ErrorContext(context=f"{context} - {func.__name__}")
                enhanced_error_handler.handle_error(
                    e, context=error_context,
                    raise_error=raise_error, log_error=log_error
                )
                if not raise_error:
                    return None

        @wraps(func)
        def sync_wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                error_context = ErrorContext(context=f"{context} - {func.__name__}")
                enhanced_error_handler.handle_error(
                    e, context=error_context,
                    raise_error=raise_error, log_error=log_error
                )
                if not raise_error:
                    return None

        # 根据函数是否为异步函数返回相应的包装器
        import inspect
        if inspect.iscoroutinefunction(func):
            return async_wrapper
        else:
            return sync_wrapper

    return decorator


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
测试日志文件的生成时机
"""

import os
import sys
import time
from pathlib import Path

# 添加项目根目录到Python路径
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))

from crawlo.logging import configure_logging as configure, get_logger


def test_log_file_generation_timing():
    """测试日志文件的生成时机"""
    print("=== 测试日志文件的生成时机 ===")
    
    # 设置日志文件路径
    log_file = "logs/timing_test.log"
    
    # 确保日志目录存在
    os.makedirs("logs", exist_ok=True)
    
    # 删除可能存在的旧日志文件
    if os.path.exists(log_file):
        os.remove(log_file)
        print(f"已删除旧日志文件: {log_file}")
    
    print(f"日志文件路径: {log_file}")
    print(f"日志文件是否存在: {os.path.exists(log_file)}")
    
    # 配置日志系统
    print("\n1. 配置日志系统...")
    configure(
        level='INFO',
        file_path=log_file,
        console_enabled=True,
        file_enabled=True
    )
    
    print(f"配置后日志文件是否存在: {os.path.exists(log_file)}")
    
    # 获取logger
    print("\n2. 获取logger...")
    logger = get_logger('timing.test')
    print(f"获取logger后日志文件是否存在: {os.path.exists(log_file)}")
    
    # 立即写入日志
    print("\n3. 立即写入日志...")
    logger.info("立即写入的第一条日志")
    print(f"写入第一条日志后日志文件是否存在: {os.path.exists(log_file)}")
    
    # 检查文件内容
    if os.path.exists(log_file):
        with open(log_file, 'r', encoding='utf-8') as f:
            content = f.read()
            print(f"日志文件内容行数: {len(content.splitlines())}")
            print(f"日志文件大小: {os.path.getsize(log_file)} 字节")
    
    # 等待一小段时间
    print("\n4. 等待1秒...")
    time.sleep(1)
    print(f"等待后日志文件是否存在: {os.path.exists(log_file)}")
    
    # 再写入一些日志
    print("\n5. 再写入一些日志...")
    for i in range(5):
        logger.info(f"第{i+1}条测试日志")
        time.sleep(0.1)
    
    print(f"写入多条日志后日志文件是否存在: {os.path.exists(log_file)}")
    
    # 检查最终文件内容
    if os.path.exists(log_file):
        with open(log_file, 'r', encoding='utf-8') as f:
            lines = f.readlines()
            print(f"\n最终日志文件内容:")
            print(f"  总行数: {len(lines)}")
            print(f"  文件大小: {os.path.getsize(log_file)} 字节")
            if lines:
                print(f"  第一行: {lines[0].strip()}")
                print(f"  最后一行: {lines[-1].strip()}")
    
    print("\n=== 测试完成 ===")


def test_buffering_behavior():
    """测试日志缓冲行为"""
    print("\n=== 测试日志缓冲行为 ===")
    
    # 设置日志文件路径
    log_file = "logs/buffering_test.log"
    
    # 删除可能存在的旧日志文件
    if os.path.exists(log_file):
        os.remove(log_file)
    
    # 配置日志系统
    configure(
        level='INFO',
        file_path=log_file,
        console_enabled=True,
        file_enabled=True
    )
    
    logger = get_logger('buffering.test')
    
    print("1. 写入日志后立即检查文件...")
    logger.info("缓冲测试日志1")
    print(f"   写入后文件是否存在: {os.path.exists(log_file)}")
    
    if os.path.exists(log_file):
        with open(log_file, 'r', encoding='utf-8') as f:
            content = f.read()
            print(f"   文件内容: '{content.strip()}'")
    
    print("2. 强制刷新并检查...")
    # 获取文件处理器并强制刷新
    for handler in logger.handlers:
        if hasattr(handler, 'flush'):
            handler.flush()
    
    if os.path.exists(log_file):
        with open(log_file, 'r', encoding='utf-8') as f:
            content = f.read()
            print(f"   刷新后文件内容: '{content.strip()}'")


def main():
    """主函数"""
    print("开始测试日志文件的生成时机...")
    
    try:
        test_log_file_generation_timing()
        test_buffering_behavior()
        
        print("\n=== 所有测试完成 ===")
        
    except Exception as e:
        print(f"\n测试过程中出现错误: {e}")
        import traceback
        traceback.print_exc()
        return 1
    
    return 0


if __name__ == '__main__':
    sys.exit(main())


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
最终综合测试脚本
全面测试框架的所有核心功能，特别是我们修改的部分
"""

import sys
import os
import asyncio
import time
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.spider import Spider
from crawlo import Request


class TestSpider(Spider):
    """测试爬虫"""
    name = 'final_test_spider'
    
    def start_requests(self):
        """发起测试请求"""
        # 生成一些测试请求
        for i in range(5):
            yield Request(f'https://httpbin.org/get?page={i}', callback=self.parse)
    
    def parse(self, response):
        """解析响应"""
        print(f"成功获取响应: {response.url}")
        print(f"状态码: {response.status_code}")
        return []


async def test_queue_blocking_behavior():
    """测试队列阻塞行为"""
    print("测试队列阻塞行为...")
    
    from crawlo.queue.queue_manager import QueueConfig, QueueManager
    
    # 创建小队列配置
    queue_config = QueueConfig(
        queue_type='memory',
        max_queue_size=3  # 非常小的队列
    )
    
    # 创建队列管理器
    queue_manager = QueueManager(queue_config)
    await queue_manager.initialize()
    
    # 测试添加超过队列大小的请求
    print("添加6个请求到大小为3的队列...")
    start_time = time.time()
    
    # 创建生产者任务
    async def producer():
        for i in range(6):
            request = Request(f'https://example.com/test{i}')
            await queue_manager.put(request)
            print(f"添加请求 {i}")
    
    # 创建消费者任务
    async def consumer():
        retrieved = 0
        while retrieved < 6:
            request = await queue_manager.get(timeout=2.0)
            if request:
                print(f"获取请求: {request.url}")
                retrieved += 1
                await asyncio.sleep(0.1)  # 模拟处理时间
    
    # 并发运行生产者和消费者
    await asyncio.gather(producer(), consumer())
    
    end_time = time.time()
    print(f"队列测试完成，耗时 {end_time - start_time:.2f} 秒")
    
    # 关闭队列
    await queue_manager.close()


async def test_framework_initialization():
    """测试框架初始化"""
    print("测试框架初始化...")
    
    from crawlo.initialization import initialize_framework
    
    # 测试默认配置
    settings = initialize_framework()
    print(f"默认配置 - RUN_MODE: {settings.get('RUN_MODE')}")
    print(f"默认配置 - QUEUE_TYPE: {settings.get('QUEUE_TYPE')}")
    
    # 测试自定义配置
    custom_settings = {
        'PROJECT_NAME': 'final_test',
        'SCHEDULER_MAX_QUEUE_SIZE': 100
    }
    
    settings = initialize_framework(custom_settings)
    print(f"自定义配置 - PROJECT_NAME: {settings.get('PROJECT_NAME')}")
    print(f"自定义配置 - SCHEDULER_MAX_QUEUE_SIZE: {settings.get('SCHEDULER_MAX_QUEUE_SIZE')}")


async def test_crawler_execution():
    """测试爬虫执行"""
    print("测试爬虫执行...")
    
    from crawlo.initialization import initialize_framework
    from crawlo.crawler import CrawlerProcess
    
    # 初始化框架
    settings = initialize_framework({
        'PROJECT_NAME': 'final_test'
    })
    
    # 创建爬虫进程
    process = CrawlerProcess(settings=settings)
    
    # 运行爬虫
    await process.crawl(TestSpider)


async def main():
    """主函数"""
    print("开始最终综合测试...")
    print("=" * 50)
    
    try:
        # 1. 测试框架初始化
        await test_framework_initialization()
        print()
        
        # 2. 测试队列阻塞行为
        await test_queue_blocking_behavior()
        print()
        
        # 3. 测试爬虫执行
        await test_crawler_execution()
        print()
        
        print("=" * 50)
        print("所有测试通过！框架工作正常。")
        
    except Exception as e:
        print("=" * 50)
        print(f"测试失败: {e}")
        import traceback
        traceback.print_exc()


if __name__ == "__main__":
    asyncio.run(main())


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import sys
import os
sys.path.insert(0, "/Users/oscar/projects/Crawlo")
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
组件工厂测试
"""
import sys
import os
import unittest

# 添加项目根目录到 Python 路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from crawlo.factories import (
    ComponentRegistry, 
    ComponentFactory, 
    ComponentSpec, 
    DefaultComponentFactory,
    CrawlerComponentFactory,
    get_component_registry
)
from crawlo.factories.base import ComponentSpec


class TestComponent:
    """测试组件类"""
    def __init__(self, name="test_component", value=42):
        self.name = name
        self.value = value


class TestComponentFactory(unittest.TestCase):
    """组件工厂测试类"""

    def setUp(self):
        """测试前准备"""
        self.registry = ComponentRegistry()
        
    def test_component_spec_creation(self):
        """测试组件规范创建"""
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="test_component",
            component_type=TestComponent,
            factory_func=factory_func,
            dependencies=[],
            singleton=False
        )
        
        self.assertEqual(spec.name, "test_component")
        self.assertEqual(spec.component_type, TestComponent)
        self.assertEqual(spec.dependencies, [])
        self.assertFalse(spec.singleton)
        
    def test_default_component_factory(self):
        """测试默认组件工厂"""
        factory = DefaultComponentFactory()
        
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="test_component",
            component_type=TestComponent,
            factory_func=factory_func
        )
        
        # 测试创建组件
        component = factory.create(spec, name="created_component", value=100)
        self.assertIsInstance(component, TestComponent)
        self.assertEqual(component.name, "created_component")
        self.assertEqual(component.value, 100)
        
        # 测试单例模式
        spec_singleton = ComponentSpec(
            name="singleton_component",
            component_type=TestComponent,
            factory_func=factory_func,
            singleton=True
        )
        
        component1 = factory.create(spec_singleton, name="singleton_1", value=200)
        component2 = factory.create(spec_singleton, name="singleton_2", value=300)
        
        # 单例应该返回相同的实例
        self.assertIs(component1, component2)
        self.assertEqual(component1.value, 200)  # 应该是第一次创建时的值
        
    def test_component_registry_registration(self):
        """测试组件注册表注册功能"""
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="registered_component",
            component_type=TestComponent,
            factory_func=factory_func
        )
        
        # 注册组件规范
        self.registry.register(spec)
        
        # 验证注册
        retrieved_spec = self.registry.get_spec("registered_component")
        self.assertEqual(retrieved_spec, spec)
        
        # 测试列出组件
        components = self.registry.list_components()
        self.assertIn("registered_component", components)
        
    def test_component_registry_creation(self):
        """测试组件注册表创建功能"""
        def factory_func(**kwargs):
            return TestComponent(**kwargs)
            
        spec = ComponentSpec(
            name="creatable_component",
            component_type=TestComponent,
            factory_func=factory_func
        )
        
        # 注册组件规范
        self.registry.register(spec)
        
        # 创建组件
        component = self.registry.create("creatable_component", name="created", value=500)
        self.assertIsInstance(component, TestComponent)
        self.assertEqual(component.name, "created")
        self.assertEqual(component.value, 500)
        
    def test_global_component_registry(self):
        """测试全局组件注册表"""
        registry = get_component_registry()
        self.assertIsInstance(registry, ComponentRegistry)
        
        # 测试注册表是否包含预注册的组件
        components = registry.list_components()
        # 应该至少包含crawler组件
        self.assertGreater(len(components), 0)
        
    def test_crawler_component_factory(self):
        """测试Crawler组件工厂"""
        factory = CrawlerComponentFactory()
        
        # 测试支持检查
        class MockEngine:
            pass
            
        self.assertTrue(factory.supports(MockEngine))  # 默认支持所有类型
        
        # 测试创建功能（需要crawler依赖）
        def mock_engine_factory(crawler=None, **kwargs):
            if crawler is None:
                raise ValueError("需要crawler实例")
            return "mock_engine"
            
        spec = ComponentSpec(
            name="mock_engine",
            component_type=type('MockEngine', (), {}),
            factory_func=mock_engine_factory,
            dependencies=['crawler']
        )
        
        # 测试缺少依赖时的错误处理
        with self.assertRaises(ValueError):
            factory.create(spec)
            
        # 测试带依赖的创建
        result = factory.create(spec, crawler="mock_crawler")
        self.assertEqual(result, "mock_engine")


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
测试日志配置修复
"""

import os
import sys
from pathlib import Path

# 添加项目根目录到Python路径
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))

from crawlo.logging import configure_logging as configure, get_logger, LogManager
from crawlo.logging.config import LogConfig


def test_correct_configuration():
    """测试正确的配置方式"""
    print("=== 测试正确的配置方式 ===")
    
    # 1. 使用配置字典（推荐方式）
    print("1. 使用配置字典...")
    LogManager().reset()
    
    config_dict = {
        'level': 'DEBUG',
        'file_path': 'fix_test.log',
        'max_bytes': 2048,
        'backup_count': 2,
        'console_enabled': True,
        'file_enabled': True,
        'encoding': 'utf-8'
    }
    
    config = configure(**config_dict)
    print(f"   配置文件路径: {config.file_path}")
    
    logger = get_logger('test.fix')
    file_handler_found = False
    for handler in logger.handlers:
        if 'FileHandler' in type(handler).__name__:
            file_handler_found = True
            print(f"   文件处理器文件名: {handler.baseFilename}")
    
    print(f"   文件处理器找到: {file_handler_found}")
    logger.info("修复测试消息")
    
    # 2. 使用LogConfig对象
    print("2. 使用LogConfig对象...")
    LogManager().reset()
    
    log_config = LogConfig(
        level='INFO',
        file_path='object_test.log',
        max_bytes=1024,
        backup_count=1,
        console_enabled=True,
        file_enabled=True
    )
    
    config = configure(log_config)
    print(f"   配置文件路径: {config.file_path}")
    
    logger = get_logger('test.object')
    file_handler_found = False
    for handler in logger.handlers:
        if 'FileHandler' in type(handler).__name__:
            file_handler_found = True
            print(f"   文件处理器文件名: {handler.baseFilename}")
    
    print(f"   文件处理器找到: {file_handler_found}")
    logger.info("对象配置测试消息")


def create_settings_class():
    """创建一个模拟的settings类来测试from_settings方法"""
    print("\n=== 测试from_settings方法 ===")
    
    class MockSettings:
        def __init__(self):
            self.LOG_LEVEL = 'DEBUG'
            self.LOG_FILE = 'settings_test.log'
            self.LOG_MAX_BYTES = 1024
            self.LOG_BACKUP_COUNT = 2
            self.LOG_ENCODING = 'utf-8'
            self.LOG_CONSOLE_ENABLED = True
            self.LOG_FILE_ENABLED = True
            self.LOG_LEVELS = {}
    
    # 测试from_settings方法
    settings = MockSettings()
    config = LogConfig.from_settings(settings)
    
    print(f"   从settings创建的配置:")
    print(f"     级别: {config.level}")
    print(f"     文件路径: {config.file_path}")
    print(f"     轮转大小: {config.max_bytes}")
    print(f"     备份数量: {config.backup_count}")
    print(f"     控制台启用: {config.console_enabled}")
    print(f"     文件启用: {config.file_enabled}")
    
    # 应用配置
    LogManager().reset()
    configure(config)
    
    logger = get_logger('test.settings')
    file_handler_found = False
    for handler in logger.handlers:
        if 'FileHandler' in type(handler).__name__:
            file_handler_found = True
            print(f"   文件处理器文件名: {handler.baseFilename}")
    
    print(f"   文件处理器找到: {file_handler_found}")
    logger.info("Settings配置测试消息")


def main():
    """主函数"""
    print("测试日志配置修复...")
    
    try:
        test_correct_configuration()
        create_settings_class()
        
        print("\n=== 修复测试完成 ===")
        print("\n推荐的日志配置方式:")
        print("1. 使用配置字典: configure(level='DEBUG', file_path='test.log', ...)")
        print("2. 使用LogConfig对象: configure(LogConfig(...))")
        print("3. 使用settings对象: configure(settings_object)")
        
    except Exception as e:
        print(f"\n测试过程中出现错误: {e}")
        import traceback
        traceback.print_exc()
        return 1
    
    return 0


if __name__ == '__main__':
    sys.exit(main())


import asyncio
import pickle
import time
import traceback
from typing import Optional, TYPE_CHECKING, List, Union, Any, Dict, Tuple, cast
from typing import Awaitable

import redis.asyncio as aioredis

# 尝试导入Redis集群支持
try:
    from redis.asyncio.cluster import RedisCluster
    REDIS_CLUSTER_AVAILABLE = True
except ImportError:
    RedisCluster = None
    REDIS_CLUSTER_AVAILABLE = False

# 使用 TYPE_CHECKING 避免运行时循环导入
if TYPE_CHECKING:
    from crawlo import Request

from crawlo.logging import get_logger
from crawlo.utils.request_serializer import RequestSerializer
from crawlo.utils.error_handler import ErrorHandler, ErrorContext
from crawlo.utils.redis_manager import get_redis_pool, RedisConnectionPool, RedisKeyManager

# 创建logger实例
logger = get_logger(__name__)

# 延迟初始化避免循环依赖
_error_handler = None


def get_module_error_handler():
    global _error_handler
    if _error_handler is None:
        _error_handler = ErrorHandler(__name__)
    return _error_handler


class RedisPriorityQueue:
    """
    基于 Redis 的分布式异步优先级队列
    """

    def __init__(
            self,
            redis_url: Optional[str] = None,
            queue_name: Optional[str] = None,
            failed_queue: Optional[str] = None,
            max_retries: int = 3,
            timeout: int = 300,
            max_connections: int = 10,
            project_name: str = "default",
            spider_name: Optional[str] = None,
            is_cluster: bool = False,
            cluster_nodes: Optional[List[str]] = None,
    ) -> None:
        """
        初始化 Redis 优先级队列
        
        Args:
            redis_url: Redis连接URL
            queue_name: 队列名称（可选，自动生成）
            failed_queue: 失败队列名称（可选，自动生成）
            max_retries: 最大重试次数
            timeout: 超时时间（秒）
            max_connections: 最大连接数
            project_name: 项目名称
            spider_name: 爬虫名称（可选）
            is_cluster: 是否为集群模式
            cluster_nodes: 集群节点列表
        """
        # 移除直接使用 os.getenv()，要求通过参数传递 redis_url
        if redis_url is None:
            # 如果没有提供 redis_url，则抛出异常，要求在 settings 中配置
            raise ValueError("redis_url must be provided. Configure it in settings instead of using os.getenv()")

        self.redis_url: str = redis_url
        self.is_cluster: bool = is_cluster
        self.cluster_nodes: Optional[List[str]] = cluster_nodes

        # 创建 Redis Key 管理器
        self.key_manager = RedisKeyManager(project_name, spider_name)
        
        # 添加调试信息
        logger.debug(f"RedisPriorityQueue initialized with project_name: {project_name}, spider_name: {spider_name}")

        # 如果未提供 queue_name，则根据 key_manager 自动生成
        self.queue_name = queue_name or self.key_manager.get_requests_queue_key()

        # 如果未提供 failed_queue，则根据 key_manager 自动生成
        self.failed_queue = failed_queue or self.key_manager.get_failed_queue_key()

        self.max_retries: int = max_retries
        self.timeout: int = timeout
        self.max_connections: int = max_connections
        self._redis_pool: Optional[RedisConnectionPool] = None
        self._redis: Optional[Any] = None
        self._lock: asyncio.Lock = asyncio.Lock()
        self.request_serializer: RequestSerializer = RequestSerializer()

    async def connect(self, max_retries: int = 3, delay: int = 1) -> Optional[Any]:
        """
        异步连接 Redis，支持重试
        
        Args:
            max_retries: 最大重试次数
            delay: 重试延迟时间（秒）
            
        Returns:
            Optional[Any]: Redis 客户端实例
        """
        async with self._lock:
            if self._redis is not None:
                # 如果已经连接，测试连接是否仍然有效
                try:
                    await self._redis.ping()
                    return self._redis
                except Exception:
                    # 连接失效，重新连接
                    self._redis = None

            for attempt in range(max_retries):
                try:
                    # 使用优化的连接池，确保 decode_responses=False 以避免编码问题
                    self._redis_pool = get_redis_pool(
                        self.redis_url,
                        is_cluster=self.is_cluster,
                        cluster_nodes=self.cluster_nodes,
                        max_connections=self.max_connections,
                        socket_connect_timeout=5,
                        socket_timeout=30,
                        health_check_interval=30,
                        retry_on_timeout=True,
                        decode_responses=False,  # 确保不自动解码响应
                        encoding='utf-8'
                    )

                    self._redis = await self._redis_pool.get_connection()

                    # 测试连接
                    if self._redis:
                        await self._redis.ping()
                    return self._redis
                except Exception as e:
                    error_msg = f"Redis 连接失败 (尝试 {attempt + 1}/{max_retries}, Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {e}"
                    logger.warning(error_msg)
                    logger.debug(f"详细错误信息:\n{traceback.format_exc()}")
                    if attempt < max_retries - 1:
                        await asyncio.sleep(delay)
                    else:
                        raise ConnectionError(f"无法连接 Redis (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {e}")

    async def _ensure_connection(self) -> None:
        """确保连接有效"""
        if self._redis is None:
            await self.connect()
        try:
            if self._redis:
                await self._redis.ping()
        except Exception as e:
            logger.warning(f"Redis 连接失效 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})，尝试重连...: {e}")
            self._redis = None
            await self.connect()

    def _is_cluster_mode(self) -> bool:
        """
        检查是否为集群模式
        
        Returns:
            bool: 是否为集群模式
        """
        if REDIS_CLUSTER_AVAILABLE and RedisCluster is not None:
            # 检查 _redis 是否为 RedisCluster 实例
            if self._redis is not None and isinstance(self._redis, RedisCluster):
                return True
        return False

    def _execute_with_cluster_support(self, operation_func, *args, **kwargs):
        """
        执行支持集群模式的操作
        
        Args:
            operation_func: 要执行的操作函数
            *args: 位置参数
            **kwargs: 关键字参数
            
        Returns:
            操作函数的返回结果
        """
        # 确保连接有效
        if self._redis is None:
            raise RuntimeError("Redis连接未初始化")
            
        # 根据是否为集群模式执行操作
        if self._is_cluster_mode():
            return operation_func(cluster_mode=True, *args, **kwargs)
        else:
            return operation_func(cluster_mode=False, *args, **kwargs)

    async def put(self, request: 'Request', priority: int = 0) -> bool:
        """
        放入请求到队列
        
        Args:
            request: 请求对象
            priority: 优先级
            
        Returns:
            bool: 是否成功放入队列
        """
        try:
            await self._ensure_connection()
            if not self._redis:
                return False
                
            # 修复优先级行为一致性问题
            # 原来: score = -priority （导致priority大的先出队）
            # 现在: score = priority （确保priority小的先出队，与内存队列一致）
            score = priority
            key = self._get_request_key(request)

            # 🔥 使用专用的序列化工具清理 Request
            clean_request = self.request_serializer.prepare_for_serialization(request)

            # 确保序列化后的数据可以被正确反序列化
            try:
                serialized = pickle.dumps(clean_request)
                # 验证序列化数据可以被反序列化
                pickle.loads(serialized)
            except Exception as serialize_error:
                logger.error(f"请求序列化验证失败 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {serialize_error}")
                return False

            # 处理集群模式下的操作
            if self._is_cluster_mode():
                # 在集群模式下，确保所有键都在同一个slot中
                # 可以通过在键名中添加相同的哈希标签来实现
                hash_tag = "{queue}"  # 使用哈希标签确保键在同一个slot
                queue_name_with_tag = f"{self.queue_name}{hash_tag}"
                data_key_with_tag = self.key_manager.get_requests_data_key() + hash_tag
                
                pipe = self._redis.pipeline()
                pipe.zadd(queue_name_with_tag, {key: score})
                pipe.hset(data_key_with_tag, key, serialized)
                result = await pipe.execute()
            else:
                pipe = self._redis.pipeline()
                pipe.zadd(self.queue_name, {key: score})
                pipe.hset(self.key_manager.get_requests_data_key(), key, serialized)
                result = await pipe.execute()

            if result[0] > 0:
                logger.debug(f"成功入队 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {request.url}")
            return result[0] > 0
        except Exception as e:
            error_context = ErrorContext(
                context=f"放入队列失败 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})"
            )
            ErrorHandler(__name__).handle_error(
                e,
                context=error_context,
                raise_error=False
            )
            return False

    async def get(self, timeout: float = 5.0) -> Optional['Request']:
        """
        获取请求（带超时）
        
        Args:
            timeout: 最大等待时间（秒），避免无限轮询
            
        Returns:
            Optional[Request]: 请求对象或 None
        """
        try:
            await self._ensure_connection()
            if not self._redis:
                return None
                
            start_time = asyncio.get_event_loop().time()

            while True:
                # 尝试获取任务
                if self._is_cluster_mode():
                    # 集群模式处理
                    hash_tag = "{queue}"
                    queue_name_with_tag = f"{self.queue_name}{hash_tag}"
                    result = await self._redis.zpopmin(queue_name_with_tag, count=1)
                else:
                    result = await self._redis.zpopmin(self.queue_name, count=1)
                    
                if result:
                    key, score = result[0]
                    data_key = self.key_manager.get_requests_data_key()
                    if self._is_cluster_mode():
                        hash_tag = "{queue}"
                        data_key = self.key_manager.get_requests_data_key() + hash_tag
                        
                    serialized = await self._redis.hget(data_key, key)
                    if not serialized:
                        continue

                    # 直接返回请求，不再移动到处理队列
                    # 更安全的反序列化方式
                    try:
                        # 首先尝试标准的 pickle 反序列化
                        request = pickle.loads(serialized)
                        return request
                    except UnicodeDecodeError:
                        # 如果出现编码错误，尝试使用 latin1 解码
                        request = pickle.loads(serialized, encoding='latin1')
                        return request
                    except Exception as pickle_error:
                        # 如果pickle反序列化失败，记录错误并跳过这个任务
                        logger.error(f"无法反序列化请求数据 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {pickle_error}")
                        # 继续尝试下一个任务
                        continue

                # 检查是否超时
                if asyncio.get_event_loop().time() - start_time > timeout:
                    return None

                # 短暂等待，避免空轮询，但减少等待时间以提高响应速度
                await asyncio.sleep(0.001)  # 从0.01减少到0.001

        except Exception as e:
            error_context = ErrorContext(
                context=f"获取队列任务失败 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})"
            )
            ErrorHandler(__name__).handle_error(
                e,
                context=error_context,
                raise_error=False
            )
            return None

    async def ack(self, request: 'Request') -> None:
        """
        确认任务完成
        
        Args:
            request: 请求对象
        """
        # 由于我们不再使用处理队列，ack方法现在是一个空操作
        # 任务在从主队列取出时就已经被认为是完成的
        logger.debug(f"任务确认完成 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {request.url}")

    async def fail(self, request: 'Request', reason: str = "") -> None:
        """
        标记任务失败
        
        Args:
            request: 请求对象
            reason: 失败原因
        """
        try:
            await self._ensure_connection()
            if not self._redis:
                return
                
            key = self._get_request_key(request)
            await self.ack(request)

            retry_key = f"{self.failed_queue}:retries:{key}"
            failed_queue = self.failed_queue
            
            if self._is_cluster_mode():
                hash_tag = "{queue}"
                retry_key = f"{self.failed_queue}:retries:{key}{hash_tag}"
                failed_queue = f"{self.failed_queue}{hash_tag}"

            retries = await self._redis.incr(retry_key)
            await self._redis.expire(retry_key, 86400)

            if retries <= self.max_retries:
                await self.put(request, priority=request.priority + 1)
                logger.info(
                    f"任务重试 [{retries}/{self.max_retries}] (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {request.url}")
            else:
                failed_data = {
                    "url": request.url,
                    "reason": reason,
                    "retries": retries,
                    "failed_at": time.time(),
                    "request_pickle": pickle.dumps(request).hex(),  # 可选：保存完整请求
                }
                await self._redis.lpush(failed_queue, pickle.dumps(failed_data))
                logger.error(f"任务彻底失败 [{retries}次] (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {request.url}")
        except Exception as e:
            error_context = ErrorContext(
                context=f"标记任务失败失败 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})"
            )
            ErrorHandler(__name__).handle_error(
                e,
                context=error_context,
                raise_error=False
            )

    def _get_request_key(self, request: 'Request') -> str:
        """
        生成请求唯一键
        
        Args:
            request: 请求对象
            
        Returns:
            str: 请求唯一键
        """
        # 使用key_manager的namespace来确保一致性
        return f"{self.key_manager.namespace}:url:{hash(request.url) & 0x7FFFFFFF}"  # 确保正数

    async def qsize(self) -> int:
        """
        Get queue size (只检查主队列)

        Returns:
            int: 队列大小（只检查主队列）
        """
        try:
            await self._ensure_connection()
            if not self._redis:
                return 0

            # 只检查主队列大小，不再检查处理中队列
            main_queue_size = 0

            if self._is_cluster_mode():
                hash_tag = "{queue}"
                queue_name_with_tag = f"{self.queue_name}{hash_tag}"
                main_queue_size = await self._redis.zcard(queue_name_with_tag)
            else:
                main_queue_size = await self._redis.zcard(self.queue_name)

            logger.debug(f"队列大小检查 - 主队列: {main_queue_size} (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})")

            return main_queue_size
        except Exception as e:
            error_context = ErrorContext(
                context=f"Failed to get queue size (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})"
            )
            get_module_error_handler().handle_error(
                e,
                context=error_context,
                raise_error=False
            )
            return 0

    async def close(self) -> None:
        """关闭连接"""
        try:
            # 显式关闭Redis连接
            if self._redis is not None:
                try:
                    # 不再自动清理Redis数据，保留数据以支持断点续爬
                    logger.debug(f"保留Redis数据以支持断点续爬 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})")
                    
                    # 尝试关闭连接
                    if hasattr(self._redis, 'close'):
                        close_result = self._redis.close()
                        if asyncio.iscoroutine(close_result):
                            await close_result
                    
                    # 等待连接关闭完成
                    if hasattr(self._redis, 'wait_closed'):
                        wait_result = self._redis.wait_closed()
                        if asyncio.iscoroutine(wait_result):
                            await wait_result
                except Exception as close_error:
                    logger.warning(
                        f"Error closing Redis connection (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name}): {close_error}"
                    )
                finally:
                    self._redis = None
            
            # 释放连接池引用（连接池由全局管理器管理）
            self._redis_pool = None
            
            logger.debug(f"Redis 连接已释放 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})")
        except Exception as e:
            error_context = ErrorContext(
                context=f"释放 Redis 连接失败 (Project: {self.key_manager.project_name}, Spider: {self.key_manager.spider_name})"
            )
            ErrorHandler(__name__).handle_error(
                e,
                context=error_context,
                raise_error=False
            )


import importlib
import pkgutil
from typing import Iterator, Any, Type, Union, Dict

from crawlo.spider import Spider


def walk_modules(module_path: str) -> Iterator[Any]:
    """
    加载模块并递归遍历其所有子模块
    
    Args:
        module_path: 模块路径
        
    Yields:
        导入的模块对象
        
    Raises:
        ImportError: 如果模块无法导入
    """
    # 导入模块
    module = importlib.import_module(module_path)
    yield module
    
    # 如果是包，则递归导入子模块
    if hasattr(module, '__path__'):
        for loader, submodule_name, is_pkg in pkgutil.walk_packages(module.__path__):
            try:
                submodule_path = f"{module_path}.{submodule_name}"
                submodule = importlib.import_module(submodule_path)
                yield submodule
                
                # 如果子模块也是包，递归遍历
                if is_pkg:
                    yield from walk_modules(submodule_path)
            except ImportError:
                # 跳过无法导入的子模块
                continue


def iter_spider_classes(module) -> Iterator[Type[Spider]]:
    """
    遍历模块中的所有Spider子类
    
    Args:
        module: 要遍历的模块
        
    Yields:
        Spider子类
    """
    for attr_name in dir(module):
        attr_value = getattr(module, attr_name)
        if (isinstance(attr_value, type) and
                issubclass(attr_value, Spider) and
                attr_value != Spider and
                hasattr(attr_value, 'name')):
            yield attr_value


def load_object(path: str):
    """
    从路径加载对象
    
    Args:
        path: 对象路径，格式为 module.submodule:object_name 或 module.submodule.object_name
        
    Returns:
        加载的对象
    """
    try:
        # 处理 module.submodule:object_name 格式
        if ':' in path:
            module_path, obj_name = path.split(':', 1)
            module = importlib.import_module(module_path)
            return getattr(module, obj_name)
        else:
            # 处理 module.submodule.object_name 格式
            module_path, obj_name = path.rsplit('.', 1)
            module = importlib.import_module(module_path)
            return getattr(module, obj_name)
    except (ImportError, AttributeError) as e:
        raise ImportError(f"Could not load object from path '{path}': {e}")


def safe_get_config(settings, key, default=None, value_type=None):
    """
    安全获取配置值的辅助函数
    
    Args:
        settings: 配置对象
        key: 配置键名
        default: 默认值
        value_type: 值类型（int, float, bool等）
        
    Returns:
        配置值或默认值
    """
    if settings is None:
        return default
        
    try:
        # 优先使用配置对象的方法
        if hasattr(settings, 'get') and callable(getattr(settings, 'get', None)):
            value = settings.get(key, default)
        # 其次处理字典类型
        elif isinstance(settings, dict):
            value = settings.get(key, default)
        # 最后尝试直接属性访问
        else:
            value = getattr(settings, key, default)
        
        # 如果指定了类型，进行类型转换
        if value_type and value is not None:
            if value_type == int:
                return int(value)
            elif value_type == float:
                return float(value)
            elif value_type == bool:
                # 特殊处理bool类型，支持字符串"0"/"1"和数字0/1的转换
                if isinstance(value, str):
                    return value.lower() not in ('0', 'false', 'no', 'off', '')
                elif isinstance(value, (int, float)):
                    return bool(value)
                else:
                    return bool(value)
        
        return value
    except (TypeError, ValueError, AttributeError, Exception):
        return default


#!/usr/bin/python
# -*- coding:UTF-8 -*-
"""
Pipeline 模块
=============

Pipeline体系：
- BasePipeline: 基础抽象类，定义Pipeline接口规范
- ResourceManagedPipeline: 提供资源管理功能（推荐使用）
- FileBasedPipeline/DatabasePipeline/CacheBasedPipeline: 特定场景的专用基类

内置去重Pipeline：
- MemoryDedupPipeline: 基于内存的去重
- RedisDedupPipeline: 基于Redis的分布式去重
- BloomDedupPipeline: 基于Bloom Filter的高效去重
- DatabaseDedupPipeline: 基于数据库的去重

使用示例：
    # 在settings.py中配置
    PIPELINES = [
        'crawlo.pipelines.RedisDedupPipeline',
        'your_project.pipelines.MongoPipeline',
    ]
"""

# 导入所有基类（从base_pipeline.py）
from .base_pipeline import (
    BasePipeline,
    ResourceManagedPipeline,
    FileBasedPipeline,
    DatabasePipeline,
    CacheBasedPipeline
)

# 导出去重管道
from .memory_dedup_pipeline import MemoryDedupPipeline
from .redis_dedup_pipeline import RedisDedupPipeline
from .bloom_dedup_pipeline import BloomDedupPipeline
from .database_dedup_pipeline import DatabaseDedupPipeline

__all__ = [
    # 基类
    'BasePipeline',
    'ResourceManagedPipeline',
    'FileBasedPipeline', 
    'DatabasePipeline',
    'CacheBasedPipeline',
    # 去重管道
    'MemoryDedupPipeline',
    'RedisDedupPipeline', 
    'BloomDedupPipeline',
    'DatabaseDedupPipeline'
]